//=========================================================================
//
//	FILE:				FTP_App.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_FTP demo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 7 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:42 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// Precompile header support
#include "CP_Precomp.h"

	// FTP demo
#include "FTP_App.h"

	// CPLAT_Framework
#include "CP_Alerts.h"
#include "CP_Application.tmpl.h"
#include "CP_BevelButton.h"
#include "CP_Dialog.h"
#include "CP_DocumentFactoryManager.h"
#include "CP_EditField.h"
#include "CP_Exceptions.h"
#include "CP_Scroller.h"
#include "CP_Separator.h"
#include "CP_StaticText.h"
#include "CP_TV_DataBrowser.h"
#include "CP_TV_DataBrowserHandlers.h"
#include "CP_TV_DataBrowserRow.h"
#include "CP_TV_DataBrowser_Defines.h"
#include "CP_Utility.h"
#include "CP_Window.h"
#include "CP_WindowManager.h"

	// local file defines
#define kCP_Internet_FTP									1000

const UInt32 kCP_RootDirectoryClickedCommand				= 1000;
const UInt32 kCP_UpDirectoryClickedCommand					= 1001;

	// ViewIDs
	// Get site dialog
const UInt32 kCP_GetSiteDialog_URLEditViewID				= 1;
const UInt32 kCP_GetSiteDialog_UserEditViewID				= 2;
const UInt32 kCP_GetSiteDialog_PasswordEditViewID			= 3;

	// Main window
const UInt32 kCP_MainWindow_HostLabelViewID					= 1;
const UInt32 kCP_MainWindow_StatusLabelViewID				= 2;
const UInt32 kCP_MainWindow_DirectoryLabelViewID			= 3;
const UInt32 kCP_MainWindow_DirectoryListingTableViewID		= 4;

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	FTP_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
FTP_Application::FTP_Application()
	: fFTPSocket(NULL), fFTPProtocol(NULL), fFTPDisplayWindow(NULL)
{
	fFTPSocket = CP_new CPLAT::CP_Socket;
	CPLAT::CP_ThrowIfNULL(fFTPSocket);
	
	fFTPProtocol = CP_new CPLAT::CP_FTP( fFTPSocket );

	fFTPProtocol->FindSignal( kCP_Signal_InternetProgress )->connect( CPLAT::CP_SigSlot::slot(this, &FTP_Application::UpdateProgress) );

	fDataSource  = CP_new CPLAT::CP_DataSource_Strings;
}

//=========================================================================
//	FUNCTION:	FTP_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
FTP_Application::~FTP_Application() throw()
{
	try
	{
		if( fFTPProtocol != NULL )
			delete fFTPProtocol;
			
		if( fFTPSocket != NULL )
		{
			delete fFTPSocket;
			fFTPSocket = NULL;
		}
	}
	catch(...)
	{
		if( fFTPSocket != NULL )
		{
			delete fFTPSocket;
			fFTPSocket = NULL;
		}
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === AboutBox ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetAboutBoxDescription
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Return the string that describes the purpose of the application in the
*	AboutBox.
*	
*	@exception	none
*
*	@return		CP_String const		The description, displayed in the about box.
*/
CPLAT::CP_String FTP_Application::GetAboutBoxDescription() const
{
	CPLAT::CP_String		returnDescription;
	
	returnDescription = "A simple CPLAT application to demonstrate the CP_FTP class.";
	
	return( returnDescription );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Commands ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleNewCommand
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Override the New command handler to create new windows. Since this application
*	is using a NULL document policy, we don't normally have the New menu item
*	enabled.
*
*	@return		inOutStatus			Return the OSStatus of this operation here.
*	@return		inCommandNo			The command being executed, in this case
*									CP_CMD_New/
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::HandleNewCommand( CPLAT::OSStatus& inOutStatus,
	CPLAT::CP_CommandNo& /*inCommandNo*/ )
{
	CPLAT::OSStatus			status = CP_noErr;
	
	HandleOpenApplication();

	inOutStatus = status;
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === GetSiteDialog ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetSiteDialog
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Present the user with a dialog to specify the ftp site to log onto.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::GetSiteDialog()
{
	try
	{
		CPLAT::CP_Dialog					*dialog = CP_new CPLAT::CP_Dialog;
		CPLAT::CP_Point						upperLeft( 50, 50 );
		CPLAT::CP_Size						contentSize( 350, 150 );
		CPLAT::CP_String					title( "Goto FTP Site" );
		CPLAT::CP_ModelessDialogWindowType	dialogWindowType;
		CPLAT::CP_Window					*dialogWindow = NULL;
		CPLAT::CP_String					okButtonTitle( "OK" );
		CPLAT::CP_String					cancelButtonTitle( "Cancel" );
		CPLAT::CP_Point						paneLocation( 0, 0 );
		CPLAT::CP_Size						paneSize( 500, 400 );
		CPLAT::CP_StaticText				*siteLabel = NULL;
		CPLAT::CP_StaticText				*userLabel = NULL;
		CPLAT::CP_StaticText				*passwordLabel = NULL;
		CPLAT::CP_String					viewTitle;
		CPLAT::CP_EditField					*siteEdit = NULL;
		CPLAT::CP_EditField					*userEdit = NULL;
		CPLAT::CP_EditField					*passwordEdit = NULL;
		
		dialog->Create( upperLeft, contentSize, title, dialogWindowType );
		dialog->CreateButtons( okButtonTitle, cancelButtonTitle );
		
		dialogWindow = dialog->GetWindow();
		
		paneLocation.Set( 13, 13 + 3 );
		paneSize.Set( 75, 16 );
		viewTitle = "Site:";
		siteLabel = CP_new CPLAT::CP_StaticText( dialogWindow, paneLocation, paneSize, viewTitle );

		paneLocation.Set( 13, 38 + 3 );
		paneSize.Set( 75, 16 );
		viewTitle = "User:";
		userLabel = CP_new CPLAT::CP_StaticText( dialogWindow, paneLocation, paneSize, viewTitle );

		paneLocation.Set( 13, 63 + 3 );
		paneSize.Set( 75, 16 );
		viewTitle = "Password:";
		passwordLabel = CP_new CPLAT::CP_StaticText( dialogWindow, paneLocation, paneSize, viewTitle );

		paneLocation.Set( 100, 13 );
		paneSize.Set( 200, 22 );
		viewTitle = "";
		siteEdit = CP_new CPLAT::CP_EditField( dialogWindow, paneLocation, paneSize, viewTitle, true );
		siteEdit->SetID( kCP_GetSiteDialog_URLEditViewID );
		
		paneLocation.Set( 100, 38 );
		paneSize.Set( 200, 22 );
		viewTitle = "";
		userEdit = CP_new CPLAT::CP_EditField( dialogWindow, paneLocation, paneSize, viewTitle, true );
		userEdit->SetID( kCP_GetSiteDialog_UserEditViewID );

		paneLocation.Set( 100, 63 );
		paneSize.Set( 200, 22 );
		viewTitle = "";
		passwordEdit = CP_new CPLAT::CP_EditField( dialogWindow, paneLocation, paneSize, viewTitle, true );
		passwordEdit->SetID( kCP_GetSiteDialog_PasswordEditViewID );

		CPLAT::CP_DataValueSignal			*dismissSignal = dialog->FindSignal( kCP_Signal_DismissDialog );
		CPLAT::CP_DataValueSignal			*cancelSignal = dialog->FindSignal( kCP_Signal_CancelDialog );

		if( dismissSignal != NULL )
			dismissSignal->connect( CPLAT::CP_SigSlot::slot(this, &FTP_Application::HandleDismiss) );
			
		if( cancelSignal != NULL )
			cancelSignal->connect( CPLAT::CP_SigSlot::slot(this, &FTP_Application::HandleCancel) );

		dialog->Run();
	}
	catch(...)
	{
	}
}

//=========================================================================
//	FUNCTION:	HandleDismiss
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot to handle the dismiss button from the GoToFTP site dialog.
*
*	@param		inValue					Contains the sending dialog.
*	@param		inOutStatus				Returns the status of the dismiss action.
*										We return CP_noErr to allow the dialog
*										to be dismissed.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::HandleDismiss( CPLAT::CP_DataValueHolder& inValue,
	CPLAT::OSStatus& inOutStatus )
{
	CPLAT::CP_Dialog			*sendingDialog = NULL;
	CPLAT::CP_Window			*dialogWindow = NULL;
	
	inValue.Get( sendingDialog );
	
	if( sendingDialog != NULL )
	{
		dialogWindow = sendingDialog->GetWindow();
		
		if( dialogWindow != NULL )
		{
			CPLAT::CP_EditField					*siteEdit = NULL;
			CPLAT::CP_EditField					*userEdit = NULL;
			CPLAT::CP_EditField					*passwordEdit = NULL;
			CPLAT::CP_String					siteString;
			CPLAT::CP_String					userString;
			CPLAT::CP_String					passwordString;
			CPLAT::CP_String					directory;
		
			siteEdit = dynamic_cast<CPLAT::CP_EditField *>(dialogWindow->FindSubView( kCP_GetSiteDialog_URLEditViewID, true ));
			userEdit = dynamic_cast<CPLAT::CP_EditField *>(dialogWindow->FindSubView( kCP_GetSiteDialog_UserEditViewID, true ));
			passwordEdit = dynamic_cast<CPLAT::CP_EditField *>(dialogWindow->FindSubView( kCP_GetSiteDialog_PasswordEditViewID, true ));

			if( siteEdit != NULL )
				siteString = siteEdit->GetText();	
			if( userEdit != NULL )
				userString = userEdit->GetText();	
			if( passwordEdit != NULL )
				passwordString = passwordEdit->GetText();	

				// try to connect to our site
			Connect( siteString, userString, passwordString );
			
				// get the initial directory
			directory = fFTPProtocol->GetWorkingDirectory();
			
				// and update the labels
			if( fFTPDisplayWindow != NULL )
			{
				CPLAT::CP_StaticText				*hostLabel = NULL;
				CPLAT::CP_StaticText				*statusLabel = NULL;
				CPLAT::CP_String					statusString( "Connected" );
				
				hostLabel = dynamic_cast<CPLAT::CP_StaticText *>(fFTPDisplayWindow->FindSubView( kCP_MainWindow_HostLabelViewID, true ));
			
				if( hostLabel != NULL )
					hostLabel->SetTitle( siteString );

				statusLabel = dynamic_cast<CPLAT::CP_StaticText *>(fFTPDisplayWindow->FindSubView( kCP_MainWindow_StatusLabelViewID, true ));
			
				if( statusLabel != NULL )
					statusLabel->SetTitle( statusString );
					
				UpdateDisplayedDirectoryLabel( directory );
				
				UpdateDirectoryListing();
			}
		}
	}
	
	inOutStatus = CP_noErr;		// so dialog is dismissed
}

//=========================================================================
//	FUNCTION:	HandleCancel
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot to handle the cancel button from the GoToFTP site dialog. This will
*	just cause the dialog to go away.
*
*	@param		inValue					Not used.
*	@param		inOutStatus				Returns the status of the cancel action.
*										We return CP_noErr to allow the dialog
*										to be dismissed.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::HandleCancel( CPLAT::CP_DataValueHolder& /*inValue*/,
	CPLAT::OSStatus& inOutStatus )
{
	inOutStatus = CP_noErr;		// so dialog is dismissed
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Main Window ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleOpenApplication
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the application is starting up. We take this opportunity to
*	create our initial window and views. In this demo, we create a the main window,
*	and display the go to site dialog.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::HandleOpenApplication()
{
	MainWindow();

		// ask user for log in info
	GetSiteDialog();
}

//=========================================================================
//	FUNCTION:	UpdateDisplayedDirectoryLabel
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the directory being displayed changes. We update the static
*	text field here with the new directory.
*
*	@param		inDirectory				String containing the new directory to
*										be displayed.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::UpdateDisplayedDirectoryLabel( const CPLAT::CP_String& inDirectory )
{
	if( fFTPDisplayWindow != NULL )
	{
		CPLAT::CP_StaticText				*directoryLabel = NULL;

		directoryLabel = dynamic_cast<CPLAT::CP_StaticText *>(fFTPDisplayWindow->FindSubView( kCP_MainWindow_DirectoryLabelViewID, true ));
	
		if( directoryLabel != NULL )
			directoryLabel->SetTitle( inDirectory );
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === FTP ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	Connect
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Establish a connection with our FTP server.
*
*	@param		inHostURL				The host URL to connect to.
*	@param		inUser					The username.
*	@param		inPassword				The password for the site.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::Connect( const CPLAT::CP_String& inHostURL,
	const CPLAT::CP_String& inUser, const CPLAT::CP_String& inPassword)
{
	try
	{		
		fFTPProtocol->Connect( inHostURL );
		fFTPProtocol->LogIn( inUser, inPassword );
	}
	catch(...)
	{
		if( fFTPProtocol != NULL )
		{
			delete fFTPProtocol;
			fFTPProtocol = NULL;
		}

		if( fFTPSocket != NULL )
		{
			delete fFTPSocket;
			fFTPSocket = NULL;
		}
		
		CPLAT::CP_String			message("FTP login error.");
		CPLAT::CP_String			description("Please check for a valid user name and password.");
		
		CPLAT::CP_Alerts::CP_OSAlert::ShowAlert( message, description );
		
		throw;		// we can't proceed
	}
}

//=========================================================================
//	FUNCTION:	MainWindow
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Displays the main window.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::MainWindow()
{
	try
	{
		CPLAT::CP_Point					windowLocation( 10, 50 );
		CPLAT::CP_Size					windowSize( 500, 400 );
		CPLAT::CP_String				windowTitle( "FTP Demo" );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		CPLAT::CP_Scroller				*tableScroller = NULL;
		CPLAT::CP_TV_DataBrowser		*tvDataBrowser = NULL;
		
		fFTPDisplayWindow = CP_new CPLAT::CP_Window();
		fFTPDisplayWindow->Create( windowLocation, windowSize, windowTitle, documentWindowType );
		
		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( fFTPDisplayWindow );
		CPLAT::CP_WindowManager::Instance()->Reposition( fFTPDisplayWindow, NULL, CPLAT::kCP_E_WindowPos_WindowCascadeOnMainScreen );

		CPLAT::CP_Point					paneLocation( 0, 0 );
		CPLAT::CP_Size					paneSize( 500, 400 );
		SInt32							counter = 1;
		SInt32							childRowCounter = 1;
		SInt32							colCounter = 1;
		CPLAT::CP_Size					scrollUnit( 20, 20 );
		CPLAT::CP_S_TV_DB_Properties	tvDataBrowserProperties;
		UInt32							nextID = 1;
		CPLAT::CP_TV_DataBrowserRow		*nextRow = NULL;
		CPLAT::CP_TV_DataBrowserRow		*childRow = NULL;
		UInt32							rowID = 1;
		CPLAT::CP_Separator				*separator = NULL;
		CPLAT::CP_StaticText			*hostLabel = NULL;
		CPLAT::CP_StaticText			*statusLabel = NULL;
		CPLAT::CP_StaticText			*directoryLabel = NULL;
		CPLAT::CP_StaticText			*hostData = NULL;
		CPLAT::CP_StaticText			*statusData = NULL;
		CPLAT::CP_StaticText			*directoryData = NULL;
		CPLAT::CP_String				viewTitle;
		CPLAT::CP_BevelButton			*rootDirectory;
		CPLAT::CP_BevelButton			*upDirectory;
		CPLAT::CP_S_ViewContentInfo		viewContentInfo;
		
			// top section -- label and data static text views
		paneLocation.Set( 5, 15 );
		paneSize.Set( 75, 16 );
		viewTitle = "Host:";
		hostLabel = CP_new CPLAT::CP_StaticText( fFTPDisplayWindow, paneLocation, paneSize, viewTitle );

		paneLocation.Set( 5, 35 );
		paneSize.Set( 75, 16 );
		viewTitle = "Status:";
		statusLabel = CP_new CPLAT::CP_StaticText( fFTPDisplayWindow, paneLocation, paneSize, viewTitle );

		paneLocation.Set( 5, 55 );
		paneSize.Set( 75, 16 );
		viewTitle = "Directory:";
		directoryLabel = CP_new CPLAT::CP_StaticText( fFTPDisplayWindow, paneLocation, paneSize, viewTitle );

		paneLocation.Set( 85, 15 );
		paneSize.Set( 300, 16 );
		viewTitle = "";
		hostData = CP_new CPLAT::CP_StaticText( fFTPDisplayWindow, paneLocation, paneSize, viewTitle );
		hostData->SetID( kCP_MainWindow_HostLabelViewID );

		paneLocation.Set( 85, 35 );
		paneSize.Set( 300, 16 );
		viewTitle = "Not Connected";
		statusData = CP_new CPLAT::CP_StaticText( fFTPDisplayWindow, paneLocation, paneSize, viewTitle );
		statusData->SetID( kCP_MainWindow_StatusLabelViewID );
		
		paneLocation.Set( 85, 55 );
		paneSize.Set( 300, 16 );
		viewTitle = "";
		directoryData = CP_new CPLAT::CP_StaticText( fFTPDisplayWindow, paneLocation, paneSize, viewTitle );
		directoryData->SetID( kCP_MainWindow_DirectoryLabelViewID );
		
		viewContentInfo.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
		
		paneLocation.Set( 5, 80 );
		paneSize.Set( 45, 24 );
		viewTitle = "/";
		rootDirectory = CP_new CPLAT::CP_BevelButton( fFTPDisplayWindow, paneLocation, paneSize, viewTitle, viewContentInfo, kCP_RootDirectoryClickedCommand );
		rootDirectory->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &FTP_Application::RootDirectoryClicked) );

		paneLocation.Set( 55, 80 );
		paneSize.Set( 45, 24 );
		viewTitle = "Up";
		upDirectory = CP_new CPLAT::CP_BevelButton( fFTPDisplayWindow, paneLocation, paneSize, viewTitle, viewContentInfo, kCP_UpDirectoryClickedCommand );
		upDirectory->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &FTP_Application::UpDirectoryClicked) );

		paneLocation.Set( 0, 117 );
		paneSize.Set( 500, 3 );
		separator = CP_new CPLAT::CP_Separator( fFTPDisplayWindow, paneLocation, paneSize );
		separator->SetFrameAdapter( CP_new CPLAT::CP_FrameAdapter_Sticky( true, true, true, false ) );

		paneLocation.Set( 0, 120 );
		paneSize.Set( 500, 400 - 120 );
		tableScroller = CP_new CPLAT::CP_Scroller( fFTPDisplayWindow, paneLocation, paneSize );
		tableScroller->SetFrameAdapter( CP_new CPLAT::CP_FrameAdapter_Sticky( true, true, true, true ) );

		paneLocation.Set( 0, 0 );
		paneSize.Set( 500 - 16, 290 - 16 );
		tvDataBrowserProperties.fNumberColumns = 1;
		tvDataBrowser = CP_new CPLAT::CP_TV_DataBrowser( tableScroller, paneLocation, paneSize, tvDataBrowserProperties );
		tvDataBrowser->SetDataSource( fDataSource );
		tvDataBrowser->SetID( kCP_MainWindow_DirectoryListingTableViewID );
		tvDataBrowser->FindSignal( kCP_Signal_DoubleClicked )->connect( CPLAT::CP_SigSlot::slot(this, &FTP_Application::DirectoryListingDoubleClick) );
		
		tableScroller->SetScrollingView( tvDataBrowser );
		tvDataBrowser->SetScrollUnit( scrollUnit );

			// set up the tables content handlers
		CPLAT::CP_S_TV_DB_ContentHandlerInfo		stringContentHandlerInfo( CPLAT::kCP_Content_String, CP_new CPLAT::CP_TV_DataBrowser_CH_String() );
		tvDataBrowser->RegisterHandler( stringContentHandlerInfo );

		tvDataBrowser->SetColumnContentType( 1, CPLAT::kCP_Content_String );
		tvDataBrowser->SetColumnWidth( -1, 1, 300 );
		
		fFTPDisplayWindow->Show();
	}
	catch(...)
	{
	}
}

//=========================================================================
//	FUNCTION:	UpdateDirectoryListing
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Updates the current directory listing.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::UpdateDirectoryListing()
{
	ASSERT( fFTPProtocol != NULL );
	
		// clear out list
	fDirectoryListingData.erase( fDirectoryListingData.begin(), fDirectoryListingData.end() );

	CPLAT::CP_TV_DataBrowser		*listingTable = NULL;
			
	listingTable = dynamic_cast<CPLAT::CP_TV_DataBrowser *>(fFTPDisplayWindow->FindSubView( kCP_MainWindow_DirectoryListingTableViewID, true ));

	if( listingTable != NULL )
	{
		listingTable->RemoveAllRows();
		listingTable->Refresh();
	}
	
	if( fFTPProtocol != NULL )
	{
		CPLAT::CP_String				fileList;
		UInt32							fileListLength;
		UInt32							x = 0;
		UInt32							y = 0;
		char							buffer[ 256 ];
		
		try
		{
			fFTPProtocol->GetList( fileList );
		}
		catch(...)
		{
			fileListLength = fileList.length();
			
			if( fileListLength == 0 )
			{
				if( fFTPDisplayWindow != NULL )
				{
					CPLAT::CP_StaticText				*statusLabel = NULL;
					CPLAT::CP_String					statusText("No files.");
					
					statusLabel = dynamic_cast<CPLAT::CP_StaticText *>(fFTPDisplayWindow->FindSubView( kCP_MainWindow_StatusLabelViewID, true ));
				
					if( statusLabel != NULL )
						statusLabel->SetTitle( statusText );
				}
			}
		}
			
			// now add our new entries
		fileListLength = fileList.length();
		
		if( fileListLength > 0 )
		{
			x = 0;
			y = 0;
			
			CPLAT::CP_Utility::CP_SetMemory( buffer, 255, '\0' );
			
				// get each string in the list and add it to our table
			for( x = 0; x < fileListLength; x++ )
			{
				char			nextChar = (char) fileList.at( x );
				
				if( nextChar != CP_LF )
				{
					if( nextChar != CP_CR )
					{
						if( y < 255 )		// limit line length to 255 characters
						{
							buffer[ y ] = nextChar;
							y++;
						}
					}
					else
					{
						if( y > 1 )		// don't list lines with/out data
						{
							buffer[ y ] = '\0';
							
							CPLAT::CP_String			nextEntry( buffer );
							fDirectoryListingData.push_back( nextEntry );
						}
						
							// get ready for next line
						CPLAT::CP_Utility::CP_SetMemory( buffer, 255, '\0' );
						y = 0;
					}
				}
			}
			
				// now add table rows for the number of entries
			UInt32							counter = 1;
			UInt32							numberRows = fDirectoryListingData.size();
			CPLAT::CP_TV_DataBrowserRow		*nextRow = 0;
			UInt32							rowID = 1;
			UInt32							colID = 1;
					
			if( listingTable != NULL )
			{
				for( counter = 1; counter <= numberRows; counter++ )
				{
					nextRow = CP_new CPLAT::CP_TV_DataBrowserRow( listingTable, rowID );
					listingTable->AddRootRow( nextRow );
					
					rowID++;

					CPLAT::CP_S_TV_DB_CellData			cellData = nextRow->GetCellData( 1 );
					cellData.fCellID = colID;
					nextRow->SetCellData( 1, cellData );
					
					colID++;
				}
				
				listingTable->Refresh();
			}
		}
	}
	
	if( fDataSource )
		fDataSource->SetStringsArray(  fDirectoryListingData );
}

//=========================================================================
//	FUNCTION:	ChangeDirectory
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Changes directory to that of the passed in directory.
*
*	@param		inDirectory				The directory to change to.
*	
*	@exception	none
*
*	@return		bool					Returns true if the change was successful.
*/
bool FTP_Application::ChangeDirectory( const CPLAT::CP_String& inDirectory )
{
	bool			result = false;
	
	try
	{
			// this line will throw an exception if the passed in inDirectory
			// is in fact not a directory
		fFTPProtocol->ChangeDirectory( inDirectory );
		
		result = true;
	}
	catch(...)
	{
		result = false;
	}
	
	return( result );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Slots ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	RootDirectoryClicked
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot called when the user clicks on a root directory in the main windows
*	list of displayed directories.
*
*	@param		inValue					Not used.
*	@param		inOutStatus				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::RootDirectoryClicked( CPLAT::CP_DataValueHolder& /*inValue*/,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
	CPLAT::CP_String			rootString("/");

	if( fFTPProtocol != NULL )
	{
		try
		{
			if( ChangeDirectory( rootString ) )
			{
				UpdateDisplayedDirectoryLabel( rootString );
				UpdateDirectoryListing();
			}
		}
		catch(...)
		{
		}
	}
}

//=========================================================================
//	FUNCTION:	UpDirectoryClicked
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot called when the user clicks on the up directory button to go to a
*	higher directory.
*
*	@param		inValue					Not used.
*	@param		inOutStatus				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::UpDirectoryClicked( CPLAT::CP_DataValueHolder& /*inValue*/,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
	if( fFTPProtocol != NULL )
	{
		try
		{
			fFTPProtocol->ChangeDirectoryUp();

			UpdateDisplayedDirectoryLabel( fFTPProtocol->GetWorkingDirectory() );
			UpdateDirectoryListing();
		}
		catch(...)
		{
		}
	}
}

//=========================================================================
//	FUNCTION:	UpdateProgress
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Updates the progress dispayed to indicate what the FTP class is doing.
*
*	@param		inValue					Contains the CP_InternetProtocolProgress
*										that indicates the progress that is
*										occurring.
*	@param		inOutStatus				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::UpdateProgress( CPLAT::CP_DataValueHolder& inValue,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
	CPLAT::CP_InternetProtocolProgress		protocolProgress;
	CPLAT::CP_String						progressString;
	
	inValue.Get( protocolProgress );

	switch( protocolProgress.GetProgressMsg() )
	{
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_OpeningConnection:
			progressString = "Opening connection";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_ConnectionDisconnected:
			progressString = "Disconnected";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_BeginAuthentication:
			progressString = "Authenticating";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_AuthenticationComplete:
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_OpeningDataConnection:
			progressString = "Opening data connection";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_DataConnectionEstablished:
			progressString = "Data connection established";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_ClosingDataConnection:
			progressString = "Closing data connection";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_BeginDataTransfer:
			progressString = "Begin data transfer";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_TransferedMoreBytes:
			progressString = "Transfering data";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_EndDataTransfer:
			progressString = "End data transfer";
			break;
		case CPLAT::CP_InternetProtocolProgress::CP_NetProto_TransferComplete:
			progressString = "Transfer complete";
			break;
	}

		// and update the display
	if( fFTPDisplayWindow != NULL )
	{
		CPLAT::CP_StaticText				*statusLabel = NULL;
		
		statusLabel = dynamic_cast<CPLAT::CP_StaticText *>(fFTPDisplayWindow->FindSubView( kCP_MainWindow_StatusLabelViewID, true ));
	
		if( statusLabel != NULL )
		{
			statusLabel->SetTitle( progressString );
		
			statusLabel->Refresh();
			
			#if TARGET_OS_MAC
				statusLabel->GetContainingWindow()->Flush();
			#endif
		}	
	}
}

//=========================================================================
//	FUNCTION:	DirectoryListingDoubleClick
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Called when the user double clicks on one of the directory listing items
*	in the main table displaying the contents of a directory.
*
*	@param		inValue					Contains the row that the user clicked on.
*	@param		inOutStatus				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void FTP_Application::DirectoryListingDoubleClick( CPLAT::CP_DataValueHolder& inValue,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
	CPLAT::CP_TV_DataBrowserRow				*clickedOnRow = NULL;
	
	inValue.Get( clickedOnRow );
	
	if( clickedOnRow != NULL )
	{
		SInt32				rowIndex = 0;

		if( fFTPDisplayWindow != NULL )
		{
			CPLAT::CP_TV_DataBrowser		*directoryListing = NULL;
			CPLAT::CP_String				clickedOnDirectory;
			
			directoryListing = dynamic_cast<CPLAT::CP_TV_DataBrowser *>(fFTPDisplayWindow->FindSubView( kCP_MainWindow_DirectoryListingTableViewID, true ));
		
			if( directoryListing != NULL )
			{
				rowIndex = directoryListing->GetRowIndex( clickedOnRow );
				
				if( rowIndex != CPLAT::kCP_InvalidIndex )
				{
					CPLAT::CP_DataValueHolder		rowData;

					if( fDataSource )
						rowData = fDataSource->GetData( rowIndex );
					
					rowData.Get( clickedOnDirectory );
					
					if( ChangeDirectory( clickedOnDirectory ) )
					{
						UpdateDisplayedDirectoryLabel( clickedOnDirectory );
						UpdateDirectoryListing();
					}
					else
					{
							// user double clicked on a file that was not a directory
					}
				}
			}
		}
	}
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
