//=========================================================================
//
//	FILE:				CP_FinderLike_TV_DataBrowser_Doc.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Document class to show the CP_TV_DataBrower in a
//						simple single column tree view.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 5 $
//						Latest Checkout:  $Date: 2005-08-30 20:33:42 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_FinderLike_TV_DataBrowser_Doc
#define _H_CP_FinderLike_TV_DataBrowser_Doc
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataSource.h"
#include "CP_Document.h"
#include "CP_File.h"
#include "CP_FileSystemSpec.h"
#include "CP_GetDirectoryDialog.h"
#include "CP_OSTypes.h"
#include "CP_PrintPolicy_None.h"
#include "CP_ScriptableObject.h"
#include "CP_STLArrayDefines.h"
#include "CP_Types.h"
#include "CP_TV_DataBrowser.h"
#include "CP_UndoPolicy_None.h"

	// Standard Library
#include <vector>

	// forward class declaration
class CP_DataSource_FileSpecs;

/*!
*	@class		CP_FinderLike_TV_DataBrowser_Doc
*	@brief		CP_Document class to handle the data from the main window and reading / writing contents.
*/
class CP_EXPORT CP_FinderLike_TV_DataBrowser_Doc : public CPLAT::CP_Document<CPLAT::CP_UndoPolicy_None, CPLAT::CP_PrintPolicy_None>
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FinderLike_TV_DataBrowser_Doc(
												CPLAT::CP_ScriptableObject *inContainer,
												CPLAT::FourCharCode inDocType );
	virtual								~CP_FinderLike_TV_DataBrowser_Doc() throw();
	//@}
			
	//! @name Creator
	//@{
	static	CPLAT::CP_Document_Base	 	*DocumentCreator(
												CPLAT::CP_ScriptableObject *inContainer,
												CPLAT::FourCharCode inDocumentClass );
	//@}

	//! @name FileSystem
	//@{
			void						GetListingDirectory();
	//@}

	//! @name Document UI
	//@{
	virtual	void						HandleCreateDocumentUI( SInt32 inPostCreateFlags = CP_E_Doc_Create_DoNothing );
	//@}

	//! @name Persist
	//@{
	virtual	void						DoPostCreate( SInt32 inPostCreateFlags = CP_E_Doc_Create_DoNothing );
	//@}

protected:

private:
	//! @name Slots
	//@{
			void						GetDirectory_Callback(
												CPLAT::CP_FileSystemDialogs_Base *inSendingDialog,
												const CPLAT::CP_FileSystemSpec& inSelection,
												bool inOverwrite );
	//@}

	//! @name Misc
	//@{
			void						PopulateTable( CPLAT::CP_TV_DataBrowser *inBrowser );
	//@}
			
	// class data
protected:
	CPLAT::CP_FileSystemSpec			fDirectory;
	CP_DataSource_FileSpecs				*fFileSpecsDataSource;
	
private:
	typedef CPLAT::CP_Document<CPLAT::CP_UndoPolicy_None, CPLAT::CP_PrintPolicy_None>	inherited;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

class CP_EXPORT CP_DataSource_FileSpecs : public CPLAT::CP_DataSource
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DataSource_FileSpecs();
	//! Destructor.
	virtual								~CP_DataSource_FileSpecs() throw();
	//@}
			
	//! @name Data
	//@{
	virtual	CPLAT::CP_DataValueHolder	GetData( SInt32 inID );
			
			SInt32						GetNumberSpecs();
			void						AddFileSpec( const CPLAT::CP_FileSystemSpec& inSpec );
	//@}

	//! @name Sorting
	//@{
	virtual	bool						Compare(
												SInt32 inIDLeft,
												SInt32 inIDRight,
												bool inSortUp );
	//@}

protected:

	// class data
protected:
	CPLAT::CP_FileSystemSpec_Array		fFileSpecsData;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#endif	// _H_CP_FinderLike_TV_DataBrowser_Doc

