//=========================================================================
//
//	FILE:				CP_UIClasses_App.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_UIClasses demo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 6 $
//						Latest Checkout:  $Date: 2005-08-28 20:41:22 -0700 (Sun, 28 Aug 2005) $
//
//=========================================================================

	// CP_UIClasses
#include "CP_UIClasses_App.h"
#include "CP_UIClasses_Defines.h"
#include "CP_UIClasses_Tab_BackView.h"

	// CPLAT_Framework
#include "CP_Alerts.h"
#include "CP_Application.tmpl.h"
#include "CP_BevelButton.h"
#include "CP_ChasingArrows.h"
#include "CP_Checkbox.h"
#include "CP_CheckGroupBox.h"
#include "CP_Clock.h"
#include "CP_Color.h"
#include "CP_DisclosureButton.h"
#include "CP_DisclosureTriangle.h"
#include "CP_EditField.h"
#include "CP_Enums.h"
#include "CP_Errors.h"
#include "CP_IconControl.h"
#include "CP_ImageWell.h"
#include "CP_KeyInputFilters.h"
#include "CP_LittleArrows.h"
#include "CP_LittleArrowsEditBuddy.h"
#include "CP_PopupGroupBox.h"
#include "CP_Pushbutton.h"
#include "CP_ProgressBar.h"
#include "CP_OSStatus.h"
#include "CP_OSTypes.h"
#include "CP_PictureControl.h"
#include "CP_Placard.h"
#include "CP_PopupArrow.h"
#include "CP_PopupMenu.h"
#include "CP_Radiobutton.h"
#include "CP_RadioGroup.h"
#include "CP_Relevance.h"
#include "CP_RoundButton.h"
#include "CP_ScrollBar.h"
#include "CP_SegmentView.h"
#include "CP_Separator.h"
#include "CP_Slider.h"
#include "CP_StaticText.h"
#include "CP_TabView.h"
#include "CP_TextGroupBox.h"
#include "CP_TextView.h"
#include "CP_Toolbar.h"
#include "CP_WindowHeader.h"
#include "CP_WindowListHeader.h"
#include "CP_WindowManager.h"
#include "CP_Utility.h"

#include "CP_SignalSlot.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_UIClasses_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
CP_UIClasses_Application::CP_UIClasses_Application()
{
	InitPopupMenuCommands();
}

//=========================================================================
//	FUNCTION:	~CP_UIClasses_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_UIClasses_Application::~CP_UIClasses_Application() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === AboutBox ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetAboutBoxDescription
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Return the string that describes the purpose of the application in the
*	AboutBox.
*	
*	@exception	none
*
*	@return		CP_String const		The description, displayed in the about box.
*/
CPLAT::CP_String CP_UIClasses_Application::GetAboutBoxDescription() const
{
	CPLAT::CP_String		returnDescription;
	
	returnDescription = "A simple CPLAT application to demonstrate basic UI elements.";
	
	return( returnDescription );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Commands ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleOpenApplication
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the application is starting up. We take this opportunity to
*	create our initial window and views. In this demo, we will open the buttons
*	window.
*	
*	@exception	none
*
*	@return		void
*/
void CP_UIClasses_Application::HandleOpenApplication()
{
	CPLAT::OSStatus			status = CP_noErr;
	CPLAT::CP_CommandNo		commandNo = CPLAT::CP_CMD_NoCommand;
	
	CreateMainWindow( status, commandNo );	
}

//=========================================================================
//	FUNCTION:	InitPopupMenuCommands
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Initialize the popup menus by populating the menu with the commands numbers
*	they will respond to. In this demo we use 2000 as the base of the command
*	numbers, and increment by the items index. These will be the commands sent
*	when the user selects an item from one of these popup menus.
*	
*	@exception	none
*
*	@return		void
*/
void CP_UIClasses_Application::InitPopupMenuCommands()
{
	UInt16									itemCounter = 0;
	
	for( itemCounter = 0; itemCounter < 5; itemCounter++ )
	{
		/*
		CPLAT::CP_CommandHandlerSignal			*popupMenuSignal = CP_new CPLAT::CP_CommandHandlerSignal();
		popupMenuSignal->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::HandlePopupCommand) );

		CPLAT::CP_CommandHandlingManager::Instance()->RegisterCommand( 2000 + itemCounter, popupMenuSignal, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );
		*/
		Connect( 2000 + itemCounter, &CP_UIClasses_Application::HandlePopupCommand, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );
	}
	
	for( itemCounter = 0; itemCounter < 3; itemCounter++ )
	{
		/*
		CPLAT::CP_CommandHandlerSignal			*popupMenuSignal = CP_new CPLAT::CP_CommandHandlerSignal();
		popupMenuSignal->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::HandlePopupCommand) );

		CPLAT::CP_CommandHandlingManager::Instance()->RegisterCommand( 2100 + itemCounter, popupMenuSignal, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );
		*/
		Connect( 2100 + itemCounter, &CP_UIClasses_Application::HandlePopupCommand, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );
	}
}

//=========================================================================
//	FUNCTION:	HandlePopupCommand
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Call when the user selects an item from one of the popup menus. In this
*	demo we will show an alert with the message "Popup menu selected" and then
*	the command number.
*
*	@return		inOutStatus			Not used.
*	@return		inCommandNo			The command number associated with the selected item.
*	
*	@exception	none
*
*	@return		void
*/
void CP_UIClasses_Application::HandlePopupCommand( CPLAT::OSStatus& /*inOutStatus*/,
	CPLAT::CP_CommandNo& inCommandNo )
{
	CPLAT::CP_String				alertMessage( "Popup menu selected: " );
	CPLAT::CP_String				alertDescription = CPLAT::CP_Utility::CP_SInt32ToString( inCommandNo );

	CPLAT::CP_Alerts::CP_OSAlert::ShowAlert( alertMessage, alertDescription );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Command Handlers ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CreateMainWindow
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Called when the application opens andn we choose New. This will create the
*	main window that contains a tab view to show the various UI elements
*
*	@return		inOutStatus			Not used.
*	@return		inCommandNo			Not used.
*	
*	@exception	none - all exceptions are trapped in this method.
*
*	@return		void
*/
void CP_UIClasses_Application::CreateMainWindow( CPLAT::OSStatus& /*inOutStatus*/,
	CPLAT::CP_CommandNo& /*inCommandNo*/ )
{
	try
	{
		CPLAT::CP_Point					windowLocation( 50, 50 );
		CPLAT::CP_Size					windowSize( 600, 300 );
		CPLAT::CP_String				windowTitle( "CPLAT UIClasses" );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		CPLAT::CP_Window				*tabWindow = NULL;
		
		tabWindow = CP_new CPLAT::CP_Window();
		tabWindow->Create( windowLocation, windowSize, windowTitle, documentWindowType );

		CPLAT::CP_WindowManager::Instance()->Reposition( tabWindow, NULL, CPLAT::kCP_E_WindowPos_WindowCascadeOnMainScreen );
		
		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( tabWindow );

		CPLAT::CP_Point					viewLocation( 20, 20 );
		CPLAT::CP_Size					viewSize( 600 - 40, 300 - 40 );
		CPLAT::CP_String				initialText( "");
		CPLAT::CP_TabView				*tabViewOne = NULL;

			// tab view
		tabViewOne = CP_new CPLAT::CP_TabView( tabWindow, viewLocation, viewSize );

		CPLAT::CP_TabInfo				tabInfo;
		CPLAT::CP_String				tabName( "Buttons" );

			// tab one
		tabInfo.fContentType.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
		tabInfo.fTabName = tabName;
		tabInfo.fEnabled = true;
		tabInfo.fView = CreateButtonView( tabViewOne );
		
		tabViewOne->AddTab( tabInfo );
		
			// tab two
		tabName = "Sliders";
		tabInfo.fContentType.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
		tabInfo.fTabName = tabName;
		tabInfo.fEnabled = true;
		tabInfo.fView = CreateTracksView( tabViewOne );
		tabInfo.fView->Hide();
		
		tabViewOne->AddTab( tabInfo );

			// tab three
		tabName = "Groups";
		tabInfo.fContentType.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
		tabInfo.fTabName = tabName;
		tabInfo.fEnabled = true;
		tabInfo.fView = CreateGroupsView( tabViewOne );
		tabInfo.fView->Hide();
		
		tabViewOne->AddTab( tabInfo );

			// tab four
		tabName = "Misc";
		tabInfo.fContentType.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
		tabInfo.fTabName = tabName;
		tabInfo.fEnabled = true;
		tabInfo.fView = CreateMiscView( tabViewOne );
		tabInfo.fView->Hide();
		
		tabViewOne->AddTab( tabInfo );

		tabWindow->Show();
	}
	catch(...)
	{
	}
}


//=========================================================================
//	FUNCTION:	CreateButtonView
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	This is used to create a CP_View that serves as the root or container view
*	for the buttons tab.
*
*	@return		inSuperview			The view that will contain this view, in this
*									case it is the tab view.
*	
*	@exception	none
*
*	@return		CP_View				The outter or containing CP_View that will contain
*									any of the subviews displayed on the content area
*									of tab one.
*/
CPLAT::CP_View *CP_UIClasses_Application::CreateButtonView( CPLAT::CP_View *inSuperview )
{
	CP_UIClasses_Tab_BackView			*returnView = NULL;
	CPLAT::CP_Point						viewLocation( 0, 30 );
	CPLAT::CP_Size						viewSize( 560, 230 );
	CPLAT::CP_String					title;
	CPLAT::CP_CheckBox					*checkboxOne = NULL;
	CPLAT::CP_CheckBox					*checkboxTwo = NULL;
	CPLAT::CP_CheckBox					*checkboxThree = NULL;
	CPLAT::CP_RadioButton				*radioButtonOne = NULL;
	CPLAT::CP_RadioButton				*radioButtonTwo = NULL;
	CPLAT::CP_PushButton				*pushButtonOne = NULL;
	CPLAT::CP_PushButton				*pushButtonTwo = NULL;
	CPLAT::CP_PushButton				*pushButtonThree = NULL;
	
		// tab view
	returnView = CP_new CP_UIClasses_Tab_BackView( inSuperview, viewLocation, viewSize );

		// push button
	viewLocation.Set( 10, 20 );
	viewSize.Set( 150, 20 );
	title = "Pushbutton";
	pushButtonOne = CP_new CPLAT::CP_PushButton( returnView, viewLocation, viewSize, title, kCP_PushButtonClickedCommand );
	pushButtonOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 170, 20 );
	viewSize.Set( 150, 20 );
	title = "Default";
	pushButtonTwo = CP_new CPLAT::CP_PushButton( returnView, viewLocation, viewSize, title, kCP_PushButtonClickedCommand, true );
	pushButtonTwo->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 330, 20 );
	viewSize.Set( 150, 20 );
	title = "Disabled";
	pushButtonThree = CP_new CPLAT::CP_PushButton( returnView, viewLocation, viewSize, title, kCP_PushButtonClickedCommand );
	pushButtonThree->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );
	pushButtonThree->Disable();
	
		// check box
	viewLocation.Set( 10, 60 );
	viewSize.Set( 150, 16 );
	title = "Checkbox one";
	checkboxOne = CP_new CPLAT::CP_CheckBox( returnView, viewLocation, viewSize, title, kCP_CheckBoxClickedCommand );
	checkboxOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 10, 80 );
	viewSize.Set( 150, 16 );
	title = "Checkbox one";
	checkboxTwo = CP_new CPLAT::CP_CheckBox( returnView, viewLocation, viewSize, title, kCP_CheckBoxClickedCommand );
	checkboxTwo->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 10, 100 );
	viewSize.Set( 150, 16 );
	title = "Checkbox one";
	checkboxThree = CP_new CPLAT::CP_CheckBox( returnView, viewLocation, viewSize, title, kCP_CheckBoxClickedCommand );
	checkboxThree->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

		// radio button
	viewLocation.Set( 10, 130 );
	viewSize.Set( 150, 16 );
	title = "Radiobutton one";
	radioButtonOne = CP_new CPLAT::CP_RadioButton( returnView, viewLocation, viewSize, title, kCP_RadioButtonClickedCommand );
	radioButtonOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 10, 150 );
	viewSize.Set( 150, 16 );
	title = "Radiobutton one";
	radioButtonTwo = CP_new CPLAT::CP_RadioButton( returnView, viewLocation, viewSize, title, kCP_RadioButtonClickedCommand );
	radioButtonTwo->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	CPLAT::CP_RadioGroup			*radioGroup1 = NULL;
	CPLAT::CP_RadioButton			*radioButtonG1 = NULL;
	CPLAT::CP_RadioButton			*radioButtonG2 = NULL;
	CPLAT::CP_RadioButton			*radioButtonG3 = NULL;
	CPLAT::CP_RadioButton			*radioButtonG4 = NULL;
	CPLAT::CP_RadioButton			*radioButtonG5 = NULL;
	
	viewLocation.Set( 200, 60 );
	viewSize.Set( 120, 100 );
	title = "Group 1";
	radioGroup1 = CP_new CPLAT::CP_RadioGroup( returnView, viewLocation, viewSize );

	viewLocation.Set( 5, 5 );
	viewSize.Set( 100, 16 );
	title = "Group 1";
	radioButtonG1 = CP_new CPLAT::CP_RadioButton( radioGroup1, viewLocation, viewSize, title, kCP_GroupRadioButton1ClickedCommand );
	radioButtonG1->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 5, 25 );
	title = "Group 2";
	radioButtonG2 = CP_new CPLAT::CP_RadioButton( radioGroup1, viewLocation, viewSize, title, kCP_GroupRadioButton2ClickedCommand );
	radioButtonG2->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 5, 45 );
	title = "Group 3";
	radioButtonG3 = CP_new CPLAT::CP_RadioButton( radioGroup1, viewLocation, viewSize, title, kCP_GroupRadioButton3ClickedCommand );
	radioButtonG3->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 5, 65 );
	title = "Group 4";
	radioButtonG4 = CP_new CPLAT::CP_RadioButton( radioGroup1, viewLocation, viewSize, title, kCP_GroupRadioButton4ClickedCommand );
	radioButtonG4->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 5, 85 );
	title = "Group 5";
	radioButtonG5 = CP_new CPLAT::CP_RadioButton( radioGroup1, viewLocation, viewSize, title, kCP_GroupRadioButton5ClickedCommand );
	radioButtonG5->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	radioGroup1->SetValue( 4 );

	CPLAT::CP_BevelButton			*bevelButtonOne = NULL;
	CPLAT::CP_BevelButton			*bevelButtonTwo = NULL;
	CPLAT::CP_BevelButton			*bevelButtonThree = NULL;
	CPLAT::CP_S_ViewContentInfo		contentInfo = { CPLAT::kCP_E_ViewContent_TextOnly };

		// bevel buttons
	viewLocation.Set( 370, 60 );
	viewSize.Set( 150, 40 );
	title = "Button 1";
	contentInfo.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
	bevelButtonOne = CP_new CPLAT::CP_BevelButton( returnView, viewLocation, viewSize, title, contentInfo, kCP_ButtonClickedCommand );
	bevelButtonOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

		// bevel button w/ icon
	viewLocation.Set( 370, 110 );
	viewSize.Set( 150, 40 );
	title = "Icon Button";
	contentInfo.fContentType = CPLAT::kCP_E_ViewContent_IconSuiteRes;
	contentInfo.u.fResourceID = 1000;
	bevelButtonTwo = CP_new CPLAT::CP_BevelButton( returnView, viewLocation, viewSize, title, contentInfo, kCP_ButtonClickedCommand + 1 );
	bevelButtonTwo->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );
	bevelButtonTwo->SetGraphicAlignment( CPLAT::kCP_E_GraphicAlign_SysDirection, 0, 0 );
	bevelButtonTwo->SetTextAlignment( CPLAT::kCP_E_TextAlign_Centered, 0 );
	
		// bevel button w/ picture
	viewLocation.Set( 370, 160 );
	viewSize.Set( 150, 40 );
	title = "Picture Button";
	
	contentInfo.fContentType = CPLAT::kCP_E_ViewContent_PictureRes;
	contentInfo.u.fResourceID = 1000;
	bevelButtonThree = CP_new CPLAT::CP_BevelButton( returnView, viewLocation, viewSize, title, contentInfo, kCP_ButtonClickedCommand + 2 );
	bevelButtonThree->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );
	bevelButtonThree->SetGraphicAlignment( CPLAT::kCP_E_GraphicAlign_AlignRight, 0, 0 );
	bevelButtonThree->SetTextPlacement( CPLAT::kCP_E_TextPlacement_Normal );

	CPLAT::CP_RoundButton			*roundButtonOne = NULL;
	CPLAT::CP_DisclosureButton		*disclosureButtonOne = NULL;
	CPLAT::CP_PopupMenu				*popupMenuOne = NULL;

		// round button
	viewLocation.Set( 10, 195 );
	viewSize.Set( 25, 25 );
	title = "Round Button";
	contentInfo.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
	roundButtonOne = CP_new CPLAT::CP_RoundButton( returnView, viewLocation, viewSize, contentInfo, kCP_RoundButtonClickedCommand );
	roundButtonOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

		// disclosure button
	viewLocation.Set( 50, 195 );
	viewSize.Set( 25, 25 );
	title = "Disclosure Button";
	disclosureButtonOne = CP_new CPLAT::CP_DisclosureButton( returnView, viewLocation, viewSize, kCP_DisclosureButtonClickedCommand );
	disclosureButtonOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

		// popup menu
	viewLocation.Set( 130, 195 );
	viewSize.Set( 200, 25 );
	title = "Popup";
	popupMenuOne = CP_new CPLAT::CP_PopupMenu( returnView, viewLocation, viewSize, title, 1000 );

	return( returnView );
}

//=========================================================================
//	FUNCTION:	CreateTracksView
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	This is used to create a CP_View that serves as the root or container view
*	for the tabs window, tracker / progresss type controls.
*
*	@return		inSuperview			The view that will contain this view, in this
*									case it is the tab view.
*	
*	@exception	none
*
*	@return		CP_View				The outter or containing CP_View that will contain
*									any of the subviews displayed on the content area
*									of tab two.
*/
CPLAT::CP_View *CP_UIClasses_Application::CreateTracksView( CPLAT::CP_View *inSuperview )
{
	CP_UIClasses_Tab_BackView			*returnView = NULL;
	CPLAT::CP_Point						viewLocation( 0, 30 );
	CPLAT::CP_Size						viewSize( 560, 230 );
	CPLAT::CP_String					title;
	
		// tab view
	returnView = CP_new CP_UIClasses_Tab_BackView( inSuperview, viewLocation, viewSize );

	CPLAT::CP_ProgressBar			*progressBarOne = NULL;
	CPLAT::CP_Slider				*sliderOne = NULL;
	CPLAT::CP_ScrollBar				*scrollBarOne = NULL;
	CPLAT::CP_EditField				*editFieldOne = NULL;
	CPLAT::CP_String				editFieldText;
	CPLAT::CP_DataValueSignal		*valueChangedSignal = NULL;
	CPLAT::CP_StaticText			*staticTextDescription = NULL;
	 
		// progress bar
	viewLocation.Set( 10, 10 );
	viewSize.Set( 200, 24 );
	progressBarOne = CP_new CPLAT::CP_ProgressBar( returnView, viewLocation, viewSize, 50, 0, 100 );

	viewLocation.Set( 10, 50 );
	viewSize.Set( 150, 16 );
	title = "Live feedback";
	staticTextDescription = CP_new CPLAT::CP_StaticText( returnView, viewLocation, viewSize, title );

		// slider
	viewLocation.Set( 10, 70 );
	viewSize.Set( 200, 24 );
	sliderOne = CP_new CPLAT::CP_Slider( returnView, viewLocation, viewSize, 0, 0, 100 );

		// scroll bar
	viewLocation.Set( 10, 110 );
	viewSize.Set( 200, 16 );
	scrollBarOne = CP_new CPLAT::CP_ScrollBar( returnView, viewLocation, viewSize, 50, 0, 100 );

	viewLocation.Set( 240, 70 );
	viewSize.Set( 50, 22 );
	editFieldOne = CP_new CPLAT::CP_EditField( returnView, viewLocation, viewSize, editFieldText, true );
	editFieldOne->AttachFilter( CP_new CPLAT::CP_KeyInputFilter_Numeric );
	
		// so we can tell if the slider or scroll bar changes values, connect up
		// to the value changed signal
	CP_Object::Connect( sliderOne, editFieldOne, true );
	CP_Object::Connect( scrollBarOne, editFieldOne, true );
	/*
	CPLAT::CP_DataValueSlot			editFieldValueSlot = editFieldOne->FindSlot( kCP_Slot_SetValue );
	valueChangedSignal = sliderOne->FindSignal( kCP_Signal_ValueChanged );
	valueChangedSignal->connect( editFieldValueSlot );

	valueChangedSignal = scrollBarOne->FindSignal( kCP_Signal_ValueChanged );
	valueChangedSignal->connect( editFieldValueSlot );
	
	CPLAT::CP_DataValueSlot			sliderSetValueSlot = sliderOne->FindSlot( kCP_Slot_SetValue );
	CPLAT::CP_DataValueSlot			scrollBarSetValueSlot = scrollBarOne->FindSlot( kCP_Slot_SetValue );

	valueChangedSignal = editFieldOne->FindSignal( kCP_Signal_ValueChanged );
	valueChangedSignal->connect( sliderSetValueSlot );
	valueChangedSignal->connect( scrollBarSetValueSlot );
	*/
		
		// set the value of the slider to 1/2 way, to update the scroll bar and edit field
	sliderOne->SetValue( 50 );

	CPLAT::CP_Slider				*sliderTwo = NULL;
	CPLAT::CP_Slider				*sliderThree = NULL;
	CPLAT::CP_Slider				*sliderFour = NULL;

	viewLocation.Set( 330, 20 );
	viewSize.Set( 24, 150 );
	sliderTwo = CP_new CPLAT::CP_Slider( returnView, viewLocation, viewSize, 0, 0, 100 );

	viewLocation.Set( 370, 20 );
	viewSize.Set( 45, 150 );
	sliderThree = CP_new CPLAT::CP_Slider( returnView, viewLocation, viewSize, 0, 0, 100, 0, 10 );

	viewLocation.Set( 430, 20 );
	viewSize.Set( 45, 150 );
	sliderFour = CP_new CPLAT::CP_Slider( returnView, viewLocation, viewSize, 0, 0, 100, CPLAT::CP_Slider::kCP_SliderDoesNotPoint );

	CPLAT::CP_Relevance				*relevanceOne = NULL;		 
	CPLAT::CP_Relevance				*relevanceTwo = NULL;		 
	CPLAT::CP_ProgressBar			*progressBarTwo = NULL;
	
	viewLocation.Set( 10, 150 );
	viewSize.Set( 100, 16 );
	relevanceOne = CP_new CPLAT::CP_Relevance( returnView, viewLocation, viewSize, 50, 0, 100 );

	viewLocation.Set( 10, 170 );
	viewSize.Set( 100, 16 );
	relevanceTwo = CP_new CPLAT::CP_Relevance( returnView, viewLocation, viewSize, 75, 0, 100 );

	viewLocation.Set( 150, 150 );
	viewSize.Set( 100, 24 );
	progressBarTwo = CP_new CPLAT::CP_ProgressBar( returnView, viewLocation, viewSize, 50, 0, 100, true );

	return( returnView );
}

//=========================================================================
//	FUNCTION:	CreateGroupsView
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	This is used to create a CP_View that serves as the root or container view
*	for the tabs window, group view.
*
*	@return		inSuperview			The view that will contain this view, in this
*									case it is the tab view.
*	
*	@exception	none
*
*	@return		CP_View				The outter or containing CP_View that will contain
*									any of the subviews displayed on the content area
*									of tab two.
*/
CPLAT::CP_View *CP_UIClasses_Application::CreateGroupsView( CPLAT::CP_View *inSuperview )
{
	CP_UIClasses_Tab_BackView			*returnView = NULL;
	CPLAT::CP_Point						viewLocation( 0, 30 );
	CPLAT::CP_Size						viewSize( 560, 230 );
	CPLAT::CP_String					title;
	
		// tab view
	returnView = CP_new CP_UIClasses_Tab_BackView( inSuperview, viewLocation, viewSize );

	CPLAT::CP_TextGroupBox			*primaryTextGroupBox = NULL;
	CPLAT::CP_TextGroupBox			*primaryGroupBox = NULL;
	CPLAT::CP_TextGroupBox			*secondaryGroupBox = NULL;
	CPLAT::CP_PopupGroupBox			*primaryPopupGroupBox = NULL;
	 
		// text group box
	viewLocation.Set( 10, 10 );
	viewSize.Set( 150, 150 );
	title = "Text group box";
	primaryTextGroupBox = CP_new CPLAT::CP_TextGroupBox( returnView, viewLocation, viewSize, title, CPLAT::kCP_GroupBox_Primary );

		// check box in group box
	CPLAT::CP_CheckBox				*textBoxCheckBoxOne = NULL;
	viewLocation.Set( 4, 20 );
	viewSize.Set( 130, 16 );
	title = "Checkbox one";
	textBoxCheckBoxOne = CP_new CPLAT::CP_CheckBox( primaryTextGroupBox, viewLocation, viewSize, title, kCP_CheckBoxClickedCommand );

		// radio button in group box
	CPLAT::CP_RadioButton			*textBoxRadioButtonOne = NULL;
	viewLocation.Set( 4, 40 );
	viewSize.Set( 130, 16 );
	title = "Radiobutton one";
	textBoxRadioButtonOne = CP_new CPLAT::CP_RadioButton( primaryTextGroupBox, viewLocation, viewSize, title, kCP_RadioButtonClickedCommand );

		// progress in group box
	CPLAT::CP_ProgressBar			*textBoxProgressOne = NULL;
	viewLocation.Set( 4, 60 );
	viewSize.Set( 100, 16 );
	textBoxProgressOne = CP_new CPLAT::CP_ProgressBar( primaryTextGroupBox, viewLocation, viewSize, 25, 0, 100 );

		// check group box
	viewLocation.Set( 200, 10 );
	viewSize.Set( 150, 200 );
	title = "Primary";
	primaryGroupBox = CP_new CPLAT::CP_TextGroupBox( returnView, viewLocation, viewSize, title, CPLAT::kCP_GroupBox_Primary );
	primaryGroupBox->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 10, 90 );
	viewSize.Set( 130, 60 );
	title = "Secondary";
	secondaryGroupBox = CP_new CPLAT::CP_TextGroupBox( primaryGroupBox, viewLocation, viewSize, title, CPLAT::kCP_GroupBox_Secondary );
	secondaryGroupBox->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

		// popup group box
	viewLocation.Set( 370, 10 );
	viewSize.Set( 150, 150 );
	title = "Popup";
	primaryPopupGroupBox = CP_new CPLAT::CP_PopupGroupBox( returnView, viewLocation, viewSize, title, 1001, 50, CPLAT::kCP_GroupBox_Primary );

		// progress in group box
	viewLocation.Set( 4, 60 );
	viewSize.Set( 100, 16 );
	textBoxProgressOne = CP_new CPLAT::CP_ProgressBar( primaryPopupGroupBox, viewLocation, viewSize, 50, 0, 100 );

	return( returnView );
}

//=========================================================================
//	FUNCTION:	CreateMiscView
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	This is used to create a CP_View that serves as the root or container view
*	for the tabs window, misc elemennts.
*
*	@return		inSuperview			The view that will contain this view, in this
*									case it is the tab view.
*	
*	@exception	none
*
*	@return		CP_View				The outter or containing CP_View that will contain
*									any of the subviews displayed on the content area
*									of tab two.
*/
CPLAT::CP_View *CP_UIClasses_Application::CreateMiscView( CPLAT::CP_View *inSuperview )
{
	CP_UIClasses_Tab_BackView			*returnView = NULL;
	CPLAT::CP_Point						viewLocation( 0, 30 );
	CPLAT::CP_Size						viewSize( 560, 230 );
	CPLAT::CP_String					title;
	
		// tab view
	returnView = CP_new CP_UIClasses_Tab_BackView( inSuperview, viewLocation, viewSize );

	CPLAT::CP_String				initialText( "" );
	CPLAT::CP_Separator				*separatorOne = NULL;		 
	CPLAT::CP_Relevance				*relevanceOne = NULL;		 
	CPLAT::CP_DisclosureTriangle	*disclosureTriangleOne = NULL;		 
	CPLAT::CP_SegmentView			*segmentViewOne = NULL;		 
	CPLAT::CP_Clock					*clockOne = NULL;		 
	CPLAT::CP_ChasingArrows			*chasingArrowsOne = NULL;
	CPLAT::CP_LittleArrows			*littleArrowsOne = NULL;
	CPLAT::CP_EditField				*editFieldOne = NULL;
	CPLAT::CP_String				editFieldText;
	CPLAT::CP_DataValueSignal		*valueChangedSignal = NULL;
	CPLAT::CP_LittleArrowsEditBuddy	*littleArrowsBuddy = NULL;
	
		// horizontal separator
	viewLocation.Set( 10, 10 );
	viewSize.Set( 100, 6 );
	separatorOne = CP_new CPLAT::CP_Separator( returnView, viewLocation, viewSize );

		// vertical separator
	viewLocation.Set( 200, 10 );
	viewSize.Set( 6, 100 );
	separatorOne = CP_new CPLAT::CP_Separator( returnView, viewLocation, viewSize );

		// disclosure triangle
	viewLocation.Set( 10, 60 );
	viewSize.Set( 120, 25 );
	initialText = "Disclosure";
	disclosureTriangleOne = CP_new CPLAT::CP_DisclosureTriangle( returnView, viewLocation, viewSize, initialText, kCP_DisclosureTriangleClickedCommand );
	//disclosureTriangleOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );
	Connect( disclosureTriangleOne, kCP_Signal_Clicked, &CP_UIClasses_Application::ControlClicked );

		// clock
	viewLocation.Set( 10, 140 );
	viewSize.Set( 100, 25 );
	clockOne = CP_new CPLAT::CP_Clock( returnView, viewLocation, viewSize );

		// chasing arrows
	viewLocation.Set( 130, 140 );
	viewSize.Set( 25, 25 );
	chasingArrowsOne = CP_new CPLAT::CP_ChasingArrows( returnView, viewLocation, viewSize );

		// little arrows
	viewLocation.Set( 170, 140 );
	viewSize.Set( 25, 25 );
	littleArrowsOne = CP_new CPLAT::CP_LittleArrows( returnView, viewLocation, viewSize, 0, 0, 100, kCP_LittleArrowsClickedCommand );
	littleArrowsOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	viewLocation.Set( 200, 140 );
	viewSize.Set( 50, 22 );
	editFieldOne = CP_new CPLAT::CP_EditField( returnView, viewLocation, viewSize, editFieldText, true );

	CPLAT::CP_DataValueSlot			editFieldValueSlot = editFieldOne->FindSlot( kCP_Slot_SetValue );
	valueChangedSignal = littleArrowsOne->FindSignal( kCP_Signal_ValueChanged );
	valueChangedSignal->connect( editFieldValueSlot );

		// little arrows
	viewLocation.Set( 10, 180 );
	viewSize.Set( 80, 25 );
	littleArrowsBuddy = CP_new CPLAT::CP_LittleArrowsEditBuddy( returnView, viewLocation, viewSize, 0, 0, 100, kCP_LittleArrowsClickedCommand );
	littleArrowsBuddy->SetValue( 50 );

		// segment view
	viewLocation.Set( 210, 30 );
	viewSize.Set( 150, 20 );
	segmentViewOne = CP_new CPLAT::CP_SegmentView( returnView, viewLocation, viewSize );
	segmentViewOne->FindSignal( CPLAT::CP_String("Clicked") )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

	CPLAT::CP_S_SegmentViewItemInfo		segmentInfo;
	
	segmentInfo.fText = "";
	segmentInfo.fColumnWidth = 25;
	segmentInfo.fCommandID = kCP_SegmentViewCol1Command;
	segmentInfo.fImageResID = 1000;
	segmentViewOne->AddItem( segmentInfo );

	segmentInfo.fText = "Two";
	segmentInfo.fColumnWidth = 50;
	segmentInfo.fCommandID = kCP_SegmentViewCol2Command;
	segmentInfo.fImageResID = 1000;
	segmentViewOne->AddItem( segmentInfo );

	segmentInfo.fText = "";
	segmentInfo.fColumnWidth = 25;
	segmentInfo.fCommandID = kCP_SegmentViewCol3Command;
	segmentInfo.fImageResID = 1000;
	segmentViewOne->AddItem( segmentInfo );

	CPLAT::CP_S_ViewContentInfo		contentInfo;
	CPLAT::CP_PictureControl		*pictureControlOne = NULL;		 
	CPLAT::CP_IconControl			*iconControlOne = NULL;		 
	CPLAT::CP_ImageWell				*imageWellOne = NULL;		 

		// picture control
	viewLocation.Set( 360, 20 );
	viewSize.Set( 50, 75 );
	contentInfo.fContentType = CPLAT::kCP_E_ViewContent_PictureRes;
	contentInfo.u.fResourceID = 1000;
	pictureControlOne = CP_new CPLAT::CP_PictureControl( returnView, viewLocation, viewSize, contentInfo, kCP_PictureClickedCommand, false );
	pictureControlOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

		// icon control
	viewLocation.Set( 360, 110 );
	viewSize.Set( 32, 32 );
	contentInfo.fContentType = CPLAT::kCP_E_ViewContent_IconSuiteRes;
	contentInfo.u.fResourceID = 1000;
	iconControlOne = CP_new CPLAT::CP_IconControl( returnView, viewLocation, viewSize, contentInfo, kCP_IconClickedCommand, false );
	iconControlOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_UIClasses_Application::ControlClicked) );

		// image well
	viewLocation.Set( 360, 170 );
	viewSize.Set( 40, 40 );
	contentInfo.fContentType = CPLAT::kCP_E_ViewContent_IconSuiteRes;
	contentInfo.u.fResourceID = 1000;
	imageWellOne = CP_new CPLAT::CP_ImageWell( returnView, viewLocation, viewSize, contentInfo );

	return( returnView );
}

//=========================================================================
//	FUNCTION:	HandleNewCommand
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Override the New command handler to create new windows. Since this application
*	is using a NULL document policy, we don't normally have the New menu item
*	enabled.
*
*	@return		inOutStatus			Return the OSStatus of this operation here.
*	@return		inCommandNo			The command being executed, in this case
*									CP_CMD_New.
*	
*	@exception	none
*
*	@return		void
*/
void CP_UIClasses_Application::HandleNewCommand( CPLAT::OSStatus& inOutStatus,
	CPLAT::CP_CommandNo& inCommandNo )
{
	CPLAT::OSStatus			status = CP_noErr;
	
	CreateMainWindow( status, inCommandNo );	

	inOutStatus = CP_noErr;
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Slots ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	ControlClicked
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Generic slot that gets called when controls are clicked. This will post an
*	alert that displies the value of the control, along with the command that
*	the sending control used when calling this slot.
*
*	@return		inValue				Holds a CP_S_ControlClickedInfo structure that
*									describes information about the sending control
*									and value from the control that was clicked.
*	@return		inOutStatus			Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_UIClasses_Application::ControlClicked( CPLAT::CP_DataValueHolder& inValue,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
	CPLAT::CP_String					description("Value: ");
	CPLAT::CP_S_ControlClickedInfo		clickedInfo;
	
	inValue.Get(clickedInfo);
		
	description += CPLAT::CP_Utility::CP_SInt32ToString( clickedInfo.fValue );
	description += "  Command: ";
	description += CPLAT::CP_Utility::CP_UInt32ToString( clickedInfo.fCommand );

		// display result to user
	CPLAT::CP_String					itemClicked;
	char								viewNameBuffer[ 1024 ];
	const char							*viewName = &viewNameBuffer[0];
	
	if( clickedInfo.fSendingControl != NULL )
	{
		CP_STD::sprintf( viewNameBuffer, "%s", typeid(*clickedInfo.fSendingControl).name() );
		itemClicked += viewName;
	}
		
	itemClicked += " clicked";
	
	CPLAT::CP_Alerts::CP_OSAlert::ShowAlert( itemClicked, description );
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32
