//=========================================================================
//
//	FILE:				CP_AutoOSHandle.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A smart pointer like class for dealing with OSHandles.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_AutoOSHandle
#define _H_CP_AutoOSHandle
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

	// Stanndard Library
#include <new>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_AutoOSHandle
*	@brief		A smart pointer like class for dealing with handles.
*	
*				This class wraps an OSs handle object into a smart pointer that manages
*				the allocation and deallocation automatically.
*/
class CP_EXPORT CP_AutoOSHandle
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AutoOSHandle();
										CP_AutoOSHandle( CP_AutoOSHandle& inAutoHandle );
	explicit							CP_AutoOSHandle( OSHandle inHandle );
	explicit							CP_AutoOSHandle( size_t inSize );
	//! Destructor.
										~CP_AutoOSHandle() throw();
	//@}
	
	//! @name operators
	//@{
			CP_AutoOSHandle&			operator = (CP_AutoOSHandle& ioHandle) throw();
			
										/*!
										*	Returns the handle as an OSHandle.
										*	
										*	@exception	none
										*	
										*	@return		OSHandle		The platforms OS handle.
										*/
										operator OSHandle () const throw()					{ return( fHandle ); }
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the handle as an OSHandle.
										*	
										*	@exception	none
										*	
										*	@return		OSHandle		The platforms OS handle.
										*/
			OSHandle					Get() const throw()									{ return( fHandle ); }
	//@}

	//! @name Management
	//@{
			OSHandle					Release() throw();

			void						Reset( OSHandle inHandle = NULL ) throw();
	//@}

protected:

	// class data
protected:
	OSHandle							fHandle;		/*! Platforms handle object.  */
	
	#if TARGET_OS_LINUX
		OSPtr							fHandlePtr;		/*! Linux pointer to the actual memory block.  */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_AutoOSHandle

