//=========================================================================
//
//	FILE:				CP_DT_MiscToolbox.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definitions of various MacOS misc toolbox functions that
//						record allocation and deallocation for leak detection.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_DT_MiscToolbox
#define _H_CP_DT_MiscToolbox
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

#if CP_Debug_RecordOSCalls

//==========================================================================
//	AppleEvents
//==========================================================================
#define AECreateDesc( typeCode, dataPtr, dataSize, result)					\
					CP_DT_AECreateDesc( typeCode, dataPtr, dataSize, result, __FUNCTION__, __FILE__, __LINE__ )
#define AEDuplicateDesc( theAEDesc, result ) 			CP_DT_AEDuplicateDesc( theAEDesc, result, __FUNCTION__, __FILE__, __LINE__ )
#define AEDisposeDesc( theAEDesc )						CP_DT_AEDisposeDesc( theAEDesc )

//==========================================================================
//	Core Foundation (CF)
//==========================================================================
#define CFStringCreateMutable( alloc, maxLength )		CP_DT_CFStringCreateMutable( alloc, maxLength, __FUNCTION__, __FILE__, __LINE__ )

#define CFRetain( cfType )								CP_DT_CFRetain( cfType, __FUNCTION__, __FILE__, __LINE__ )
#define CFRetainEvent( inEvent )						CP_DT_RetainEvent( inEvent, __FUNCTION__, __FILE__, __LINE__ )
#define CFRetainMenu( inMenu )							CP_DT_RetainMenu( inMenu, __FUNCTION__, __FILE__, __LINE__ )
#define CFRelease( cfType )								CP_DT_CFRelease( cfType )
#define CreateEvent( inAllocator, inClassID, kind, when, flags, outEvent )	CP_DT_CreateEvent( inAllocator, inClassID, kind, when, flags, outEvent, __FUNCTION__, __FILE__, __LINE__ )
#define ReleaseEvent( inEvent )							CP_DT_ReleaseEvent( inEvent )

//==========================================================================
//	GWorlds
//==========================================================================
#define NewGWorld( offscreenGWorld, PixelDepth, boundsRect, cTable, aGDevice, flags) \
					CP_DT_NewGWorld( offscreenGWorld, PixelDepth, boundsRect, cTable, \
					aGDevice, flags, __FUNCTION__, __FILE__, __LINE__ )
#define DisposeGWorld( offscreenGWorld )				CP_DT_DisposeGWorld( offscreenGWorld )

//==========================================================================
//	Menus
//==========================================================================
#define NewMenu( menuID, menuTitle )					CP_DT_NewMenu( menuID, menuTitle, __FUNCTION__, __FILE__, __LINE__ )
#define GetMenu( resourceID )							CP_DT_GetMenu( resourceID, __FUNCTION__, __FILE__, __LINE__ )
#define DisposeMenu( theMenu )							CP_DT_DisposeMenu( theMenu )
#define ReleaseMenu( theMenu )							CP_DT_ReleaseMenu( theMenu )
#define GetNewMBar( menuBarID )							CP_DT_GetNewMBar( menuBarID, __FUNCTION__, __FILE__, __LINE__ )

//==========================================================================
//	QuickDraw
//==========================================================================
#define NewRgn()										CP_DT_NewRgn( __FUNCTION__, __FILE__, __LINE__ )
#define DisposeRgn( rgn )								CP_DT_DisposeRgn( rgn )
#define OpenPicture( picFrame )							CP_DT_OpenPicture( picFrame, __FUNCTION__, __FILE__, __LINE__ )
#define KillPicture( myPicture )						CP_DT_KillPicture( myPicture )
#define NewPixMap()										CP_DT_NewPixMap( __FUNCTION__, __FILE__, __LINE__ )
#define DisposePixMap( pm )								CP_DT_DisposePixMap( pm )
#define GetCTable( ctID )								CP_DT_GetCTable( ctID, __FUNCTION__, __FILE__, __LINE__ )
#define DisposeCTable( cTable )							CP_DT_DisposeCTable( cTable )

//==========================================================================
//	Windows
//==========================================================================
#define GetNewWindow( windowID, wStorage, behind )		CP_DT_GetNewWindow( windowID, wStorage, behind, __FUNCTION__, __FILE__, __LINE__ )
#define GetNewCWindow( windowID, wStorage, behind ) 	CP_DT_GetNewCWindow( windowID, wStorage, behind, __FUNCTION__, __FILE__, __LINE__ )
#define DisposeWindow( theWindow )						CP_DT_DisposeWindow( theWindow )
#define ReleaseWindow( theWindow )						CP_DT_ReleaseWindow( theWindow )
#define CreateNewWindow( windowClass, attributes, contentBounds,outWindow )		CP_DT_CreateNewWindow( windowClass, attributes, contentBounds,outWindow, __FUNCTION__, __FILE__, __LINE__ )


	// AppleEvents
pascal OSErr		CP_DT_AECreateDesc(
							DescType typeCode,
							const void *dataPtr,
							Size  dataSize,
							AEDesc *result,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal OSErr		CP_DT_AEDuplicateDesc(
							const AEDesc *theAEDesc,
							AEDesc *result,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal OSErr 		CP_DT_AEDisposeDesc( AEDesc *theAEDesc );

	// CoreFoundation
pascal CFMutableStringRef	CP_DT_CFStringCreateMutable(
									CFAllocatorRef alloc,
  									CFIndex maxLength,
  									const char *inFunction,
									const char *inFile,
									SInt16 inLine );
pascal CFTypeRef	CP_DT_CFRetain(
									CFTypeRef cf,
  									const char *inFunction,
									const char *inFile,
									SInt16 inLine );
pascal ::OSStatus	CP_DT_CreateEvent(
									CFAllocatorRef inAllocator,
									UInt32 inClassID,
									UInt32 kind,
									EventTime when,
									EventAttributes flags,
									EventRef *outEvent,
  									const char *inFunction,
									const char *inFile,
									SInt16 inLine );
pascal EventRef		CP_DT_RetainEvent(
									EventRef inEvent,
  									const char *inFunction,
									const char *inFile,
									SInt16 inLine );
pascal MenuRef		CP_DT_RetainMenu(
									MenuRef inMenu,
  									const char *inFunction,
									const char *inFile,
									SInt16 inLine );

pascal void 		CP_DT_CFRelease( CFTypeRef cfType );
pascal void 		CP_DT_ReleaseEvent( EventRef inEvent );

	// GWorlds
pascal QDErr		CP_DT_NewGWorld(
							GWorldPtr *offscreenGWorld,
							SInt16 PixelDepth,
							const Rect *boundsRect,
							CTabHandle cTable,
							GDHandle aGDevice,
							GWorldFlags flags,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal void 		CP_DT_DisposeGWorld( GWorldPtr offscreenGWorld );

	// Menus
pascal MenuRef 		CP_DT_NewMenu(
							SInt16 menuID,
							ConstStr255Param menuTitle,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal MenuRef 		CP_DT_GetMenu(
							SInt16 resourceID,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal void 		CP_DT_DisposeMenu( MenuRef theMenu );
pascal OSStatus		CP_DT_ReleaseMenu( MenuRef theMenu );
pascal Handle 		CP_DT_GetNewMBar(
							SInt16 menuBarID,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );

	// QuickDraw
pascal RgnHandle 	CP_DT_NewRgn(
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal void 		CP_DT_DisposeRgn( RgnHandle rgn );
pascal PicHandle 	CP_DT_OpenPicture(
							const Rect *picFrame,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal void 		CP_DT_KillPicture( PicHandle myPicture );
pascal PixMapHandle CP_DT_NewPixMap(
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal void 		CP_DT_DisposePixMap( PixMapHandle pm );
pascal CTabHandle 	CP_DT_GetCTable(
							SInt16 ctID,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal void 		CP_DT_DisposeCTable( CTabHandle cTable );

	// Windows
pascal WindowRef 	CP_DT_GetNewWindow(
							SInt16 windowID,
							void *wStorage,
							WindowRef behind,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal WindowRef 	CP_DT_GetNewCWindow(
							SInt16 windowID,
							void *wStorage,
							WindowRef behind,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );
pascal void		 	CP_DT_DisposeWindow( WindowRef theWindow );
pascal OSStatus		CP_DT_ReleaseWindow( WindowRef theWindow );
pascal OSStatus 	CP_DT_CreateNewWindow(
							WindowClass  windowClass,
  							WindowAttributes attributes,
  							const Rect *contentBounds,
  							WindowRef *outWindow,
							const char *inFunction,
							const char *inFile,
							SInt16 inLine );

#endif 	// __CP_USE_DEBUG_TRAPS__

#endif	// __BUILD_MAC__

#endif	// _H_CP_DT_MiscToolbox


