//=========================================================================
//
//	FILE:				CP_DataStream.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to hold a single flavor of data from a drag
//						and drop or clipboard operation.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 247 $
//						Latest Checkout:  $Date: 2005-07-30 19:06:38 -0700 (Sat, 30 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_DataStream
#define _H_CP_DataStream
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_HandleBlock.h"
#include "CP_OSTypes.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_DataStreamItem
*	@brief		A class to hold a single flavor of data from a drag and drop or clipboard operation.
*/
class CP_EXPORT CP_DataStreamItem
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DataStreamItem();
	//! Destructor.
	virtual								~CP_DataStreamItem() throw();
	//@}
			
	//! @name Getters
	//@{
			FlavorType					GetFlavorType()								{ return( fFlavorType ); }
			UInt32						GetFlavorFlags()							{ return( fFlavorFlags ); }
			CP_HandleBlock				*GetFlavorData()							{ return( fData ); }
	//@}

	//! @name Setters
	//@{
			void						SetFlavorType( FlavorType inFlavorType )	{ fFlavorType = inFlavorType; }
			void						SetFlavorFlags( UInt32 inFlavorFlags )		{ fFlavorFlags = inFlavorFlags; }
			void						SetFlavorData( CP_HandleBlock *inData )		{ fData = inData; }
	//@}

protected:

	// class data
protected:
	FlavorType							fFlavorType;
	UInt32								fFlavorFlags;
	CP_HandleBlock						*fData;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_DataStreamItem, std::allocator<CP_DataStreamItem> >							CP_DataStreamItem_Array;
	typedef std::vector<CP_DataStreamItem, std::allocator<CP_DataStreamItem> >::iterator				CP_DataStreamItem_Iterator;
	typedef std::vector<CP_DataStreamItem, std::allocator<CP_DataStreamItem> >::reverse_iterator		CP_DataStreamItem_ReverseIterator;
#else
	typedef std::vector<CP_DataStreamItem>																CP_DataStreamItem_Array;
	typedef std::vector<CP_DataStreamItem>::iterator													CP_DataStreamItem_Iterator;
	typedef std::vector<CP_DataStreamItem>::reverse_iterator											CP_DataStreamItem_ReverseIterator;
#endif

/*!
*	@class		CP_DataStream
*	@brief		A class to hold a list of CP_DataStreamItems.
*/
class CP_EXPORT CP_DataStream
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DataStream();
										CP_DataStream( const CP_DataStream& inDataStream );
	//! Destructor.
	virtual								~CP_DataStream() throw();
	//@}

	//! @name Items
	//@{
			void						AddItem( CP_DataStreamItem& inDataStreamItem );
			CP_DataStreamItem			GetItem( UInt32 inItemIndex );
			UInt32						GetCount();
			void						ClearItems();
	//@}

	//! @name Operators
	//@{
			CP_DataStream&				operator = ( const CP_DataStream& inDataStream );
	//@}
			
protected:

	// class data
protected:
	CP_DataStreamItem_Array				*fDataStreamItems;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DataStream

