//=========================================================================
//
//	FILE:				CP_Environment.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class for dealing with the underlying OSs environment.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 199 $
//						Latest Checkout:  $Date: 2005-06-02 14:27:57 -0700 (Thu, 02 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Environment
#define _H_CP_Environment
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#if !CP_CORE_ONLY // JUM 2004-11-25
  #include "CP_MenuBar.h"
#endif
#include "CP_NoCopy.h"
#include "CP_OSTypes.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@namespace	CP_Environment
*	@brief		A set of routines to provide information about the running environment of your application.
*/
namespace CP_Environment
{
#if !CP_CORE_ONLY // JUM 2004-11-25
	enum
	{
		kCP_E_GrayscaleColor_WhiteIndex = 0,
		kCP_E_GrayscaleColor_BlackIndex = 1,
		kCP_E_GrayscaleColor_Gray1Index = 2,
		kCP_E_GrayscaleColor_Gray2Index = 3,
		kCP_E_GrayscaleColor_Gray3Index = 4,
		kCP_E_GrayscaleColor_Gray4Index = 5,
		kCP_E_GrayscaleColor_Gray5Index = 6,
		kCP_E_GrayscaleColor_Gray6Index = 7,
		kCP_E_GrayscaleColor_Gray7Index = 8,
		kCP_E_GrayscaleColor_Gray8Index = 9,
		kCP_E_GrayscaleColor_Gray9Index = 10,
		kCP_E_GrayscaleColor_Gray10Index = 11,
		kCP_E_GrayscaleColor_Gray11Index = 12,
		kCP_E_GrayscaleColor_Gray12Index = 13
	};
#endif
	
	//! @name Application Info
	//@{
		CPLAT::CP_String				GetAppName();
		FourCharCode					GetCreator();
	
		CPLAT::CP_FileSystemSpec		GetAppSpec();
		CPLAT::CP_FileSystemSpec		GetAppFolder();
	//@}
	
	//! @name OS Info
	//@{
		UInt32							GetOSVersion();
	//@}
					
	//! @name Testers
	//@{
		bool							IsOSX();
	//@}

#if !CP_CORE_ONLY // JUM 2004-11-25
	//! @name UI
	//@{
		CP_RGBColor						GetButtonColor( SInt16 inColorIndex );
	//@}
#endif

	//! @name Unicode
	//@{
		bool							HasUnicode();
	//@}

	//========================================================================
	// Platform specific routines
	//========================================================================
	#if TARGET_OS_MAC
	//! @name Features
	//@{
		bool							IsFeatureAvailable_Mac(
												OSType inSelector,
												UInt32 inAttribute );

		bool							HasCarbonVersionOrGreater( SInt32 inDesiredVersion );
		
		bool							HasMPTasks();

		Ptr								GetSharedLibrarySymbol(
												ConstStringPtr inLibraryName,
												ConstStringPtr inSymbolName );
		bool							HasSharedLibrarySymbol( UInt32 inSymbolName );
	//@}
	#endif

	#if TARGET_OS_WIN32
		bool							IsNT();
		bool							IsWin32();
		bool							IsWin32s();
		bool							IsWin98();
	#endif

	#if TARGET_OS_LINUX
	#endif

#if !CP_CORE_ONLY // JUM 2004-11-25
	/*!
	*	@class		CP_InterfaceStyle
	*	@brief		A singleton class to manage the interface style used in CPLAT applications.
	*/
	class CP_EXPORT CP_InterfaceStyle : public CP_NoCopy
	{
	public:
		enum CP_E_InterfaceStyle
		{
			CP_E_InterfaceStyle_MDI = (1 << 0),
			CP_E_InterfaceStyle_MenuBar = (1 << 1),
			CP_E_InterfaceStyle_SDI = (1 << 2)
		};

		//! @name Constructor / Destructor
		//@{
		//! Constructor.
										CP_InterfaceStyle();
		//! Destructor.
		virtual							~CP_InterfaceStyle() throw();
		//@}
				
		//! @name Instance
		//@{
		static	CP_InterfaceStyle		*Instance();
		//@}

			CP_E_InterfaceStyle			GetStyle()															{ return( fInterfaceStyle ); }
			void						SetStyle( CP_E_InterfaceStyle inMenuBarStyle )						{ fInterfaceStyle = inMenuBarStyle; }

	protected:

		// class data
	protected:
		static CP_InterfaceStyle		*fsInstance;		/*! Single instance of this class. */
		
		CP_E_InterfaceStyle				fInterfaceStyle;	/*! Type of interface used in CPLAT application. */
		
	//========================================================================
	// Platform specific routines
	//========================================================================
	#if TARGET_OS_MAC
	#endif

	#if TARGET_OS_WIN32
	public:
				void					CreateMenuBar( CP_MenuBar *inMenuBar );
	#endif

	#if TARGET_OS_LINUX
	#endif
	};
#endif // !CP_CORE_ONLY

}	// namespace CP_Environment

#if TARGET_OS_MAC
		// When using CFM programs we need to make sure the symbol is available at runtime
		// in case the symbol was just weak linked in, and we're running without the routine
		// being available.
	#if TARGET_RT_MAC_CFM
		#define IsCFMAddressResolved( inAddress )						((inAddress) != (void *) kUnresolvedCFragSymbolAddress)
	#else
		#define IsCFMAddressResolved( inAddress )						(true)
	#endif
#endif	// TARGET_OS_MAC

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Environment

