//=========================================================================
//
//	FILE:				CP_MenuBarManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A singleton class to manage 1 or more CP_MenuBars in a
//						CPLAT application.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_MenuBarManager
#define _H_CP_MenuBarManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_MenuBar.h"

CPLAT_Begin_Namespace_CPLAT

#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_MenuBar *, std::allocator<CP_MenuBar *> >							CP_MenuBars_Array;
	typedef std::vector<CP_MenuBar *, std::allocator<CP_MenuBar *> >::iterator					CP_MenuBars_Iterator;
	typedef std::vector<CP_MenuBar *, std::allocator<CP_MenuBar *> >::reverse_iterator			CP_MenuBars_ReverseIterator;
#else
	typedef std::vector<CP_MenuBar *>															CP_MenuBars_Array;
	typedef std::vector<CP_MenuBar *>::iterator													CP_MenuBars_Iterator;
	typedef std::vector<CP_MenuBar *>::reverse_iterator											CP_MenuBars_ReverseIterator;
#endif
	
/*!
*	@class	CP_MenuBarManager
*	@brief	A singleton class to manage 1 or more CP_MenuBars in a CPLAT application.
*/

class CP_EXPORT CP_MenuBarManager
{
public:
											// Construction / destruction
										CP_MenuBarManager();
	virtual								~CP_MenuBarManager();
			
	static	CP_MenuBarManager			*Instance();

											// Menu Bars
			CP_MenuBar					*LoadMenuBar( SInt16 inMenuBarID );
	
			CP_MenuBar					*GetCurrentMenuBar();

protected:

private:
			void						AddMenuBarToList( CP_MenuBar *inMenuBar );

											// Don't allow copy and assignment
										CP_MenuBarManager( const CP_MenuBarManager& );
	CP_MenuBarManager&					operator = ( const CP_MenuBarManager& );

	// class data
private:	
	static CP_MenuBarManager			*fsInstance;

	CP_MenuBars_Array					*fMenuBars;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MenuBarManager

