//=========================================================================
//
//	FILE:				CP_ObjectManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class for managing a list of CP_Objects.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_ObjectManager
#define _H_CP_ObjectManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_Object.h"

	// Standard library
#include <list>

CPLAT_Begin_Namespace_CPLAT

	// typedefs
/*!	\fn typedef CP_ObjectsList
 *	A std::list object that holds a list of CP_Objects. 
 */
#ifndef TARGET_OS_LINUX
	typedef std::list<CP_Object *, std::allocator<CP_Object *> >							CP_Objects_List;
	typedef std::list<CP_Object *, std::allocator<CP_Object *> >::iterator					CP_Objects_Iterator;
	typedef std::list<CP_Object *, std::allocator<CP_Object *> >::reverse_iterator			CP_Objects_ReverseIterator;
#else
	typedef std::list<CP_Object *>															CP_Objects_List;
	typedef std::list<CP_Object *>::iterator												CP_Objects_Iterator;
	typedef std::list<CP_Object *>::reverse_iterator										CP_Objects_ReverseIterator;
#endif

/*!
*	@class	CP_ObjectManager
*	@brief	A class for managing a list of CP_Objects
*/
class CP_EXPORT CP_ObjectManager : CP_NoCopy
{
public:
											// Construction / destruction
										CP_ObjectManager();
	virtual								~CP_ObjectManager();
			
	static	CP_ObjectManager			*Instance();

											// Objects list
			UInt32						Count();
			void						Append( CP_Object *inDocument );
			void						Remove( CP_Object *inDocument );
			CP_Object					*FindByID( FourCharCode inObjectID );
			
protected:

	// class data
protected:
	static CP_ObjectManager				*fsInstance;

	CP_Objects_List						*fObjects;				/*! List of all objects. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ObjectManager

