//=========================================================================
//
//	FILE:				CP_PersistManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Singleton class to manage CP_M_Persist object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_PersistManager
#define _H_CP_PersistManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_Persist.h"
#include "CP_String.h"
#include "CP_XMLDocument.h"

	// Standard Library
#include <map>
#include <typeinfo>

CPLAT_Begin_Namespace_CPLAT

/*! Callback typedef to create a CP_M_Persist object.. */
typedef CP_M_Persist		*(*CreateCallback)();

	// CP_String registration
typedef std::map<CP_String, CreateCallback, std::less<CP_String> >				CP_PersistCreator_Map;
typedef std::map<CP_String, CreateCallback, std::less<CP_String> >::iterator	CP_PersistCreator_Iterator;

typedef std::pair<const CP_String, CreateCallback>								CP_PersistCreator_Entry;

	// Mape of control kind to CPLAT class name
typedef std::map<UInt32, CP_String, std::less<UInt32> >							CP_PersistControlKind_Map;
typedef std::map<UInt32, CP_String, std::less<UInt32> >::iterator				CP_PersistControlKind_Iterator;

typedef std::pair<UInt32, const CP_String>										CP_PersistControlKind_Entry;

/*!
*	@class	CP_PersistManager
*	@brief	Singleton class to manage CP_M_Persist object.
*/
class CP_EXPORT CP_PersistManager
{
public:

											// Construction / destruction
										CP_PersistManager();
	virtual								~CP_PersistManager();

	static	CP_PersistManager			*Instance();

											// Creation
	CP_M_Persist						*Create(
												SInt16 inResourceID,
												UInt32 inResourceType,
												CP_View *inSuperview = NULL,
												bool inDocumentWindow = false );

	CP_M_Persist						*Create( CP_XMLDocument& inXMLDocument );

	CP_M_Persist						*CreateObject( const CP_String& inClassName );
									
	CP_String							FindClassName( const UInt32 inControlKind ) const;

											// Registration
	void								RegisterClass(
												const CP_String& inClassName,
												CreateCallback inCreatorFunction );

	void								RegisterControlKind(
												const UInt32 inControlKind,
												const CP_String& inClassNameString );

protected:

private:
											// Don't allow copy and assignment
										CP_PersistManager( const CP_PersistManager& );
	CP_PersistManager&					operator = ( const CP_PersistManager& );

	// class data
protected:
	static CP_PersistManager			*fsInstance;

	CP_PersistCreator_Map				*fPersistCreators;
	CP_PersistControlKind_Map			*fPersistCreatorsControlKind;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

template <class T>
class CP_EXPORT CP_PersistRegistration
{
public:
										CP_PersistRegistration( const CP_String& inClassNameString )
										{ 
											if( CP_PersistManager::Instance() != NULL )
											{
									          	CreateCallback			creatorCallback = NULL;
												
												creatorCallback = T::PersistCreator;

												if( creatorCallback != NULL )
													CP_PersistManager::Instance()->RegisterClass( inClassNameString, creatorCallback );
											}
									 	}
										~CP_PersistRegistration()					{}
};

class CP_EXPORT CP_ControlKindRegistration
{
public:
										CP_ControlKindRegistration(
												const UInt32 inControlKind,
												const CP_String& inClassNameString )
										{ 
											if( CP_PersistManager::Instance() != NULL )
											{
												CP_PersistManager::Instance()->RegisterControlKind( inControlKind, inClassNameString );
											}
									 	}
										~CP_ControlKindRegistration()					{}
};

	// Class name registration by string
#define	RegisterPersist(inClassName, classNameString)				CPLAT::CP_PersistRegistration<inClassName>::CP_PersistRegistration(classNameString)

	// Control kind to class name registration
#define	RegisterPersistControlKind(inControlKind, inClassName)		CPLAT::CP_ControlKindRegistration::CP_ControlKindRegistration(inControlKind, inClassName)

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PersistManager

