//=========================================================================
//
//	FILE:				CP_Printer_PageFormat.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class used with the printing functions to provide information
//						on the currently selected page format.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Printer_PageFormat
#define _H_CP_Printer_PageFormat
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_Rect.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_LINUX
struct CP_S_PageFormat
{
};
#endif

/*!
*	@class		CP_Printer_PageFormat
*	@brief		A class used with the printing functions to provide information
*				on the currently selected page format.
*/
class CP_EXPORT CP_Printer_PageFormat
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Printer_PageFormat();
										CP_Printer_PageFormat( const CP_Printer_PageFormat& inPageFormat );
	//! Destructor.
	virtual								~CP_Printer_PageFormat() throw();
	//@}
			
											// Getters
			CP_Size						GetPrintAreaOffset();
			CP_Rect						GetPageRect();
			CP_Rect						GetPrintAreaRect();

											// operators
			CP_Printer_PageFormat&		operator =( const CP_Printer_PageFormat& inPageFormat );

			void						swap( CP_Printer_PageFormat& inOutPageFormat ) throw();

protected:

	// class data
protected:

	#if TARGET_OS_MAC
		PMPageFormat					fPageFormat;
	#endif
	
	#if TARGET_OS_WIN32
		PAGESETUPDLG					fPageFormat;
	#endif
	
	#if TARGET_OS_LINUX
		CP_S_PageFormat					fPageFormat;
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
			explicit					CP_Printer_PageFormat( PMPageFormat inPageFormat );
	//@}

			PMPageFormat				GetPMPageFormat()									{ return( fPageFormat ); }
			void						SetPMPageFormat( PMPageFormat inPageFormat );
#endif

#if TARGET_OS_WIN32
public:
										CP_Printer_PageFormat( PAGESETUPDLG infPageFormat );

			PAGESETUPDLG				GetPrintRecord()									{ return( fPageFormat ); }
			void						SetPrintRecord( PAGESETUPDLG inPrintRecord )		{fPageFormat = inPrintRecord; }

#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Printer_PageFormat

