//=========================================================================
//
//	FILE:				CP_SignalSlot.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes to handle CP_Signals and CP_Slots in the CPLAT
//						framework.
//	
//	HISTORY:			1st Version - April 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 265 $
//						Latest Checkout:  $Date: 2005-08-29 21:52:42 -0700 (Mon, 29 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_SignalSlot
#define _H_CP_SignalSlot
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework

	// Standard Library
#include <iostream>
#include <list>
#include <set>

CPLAT_Begin_Namespace_CPLAT

namespace CP_SigSlot
{

class CP_Signal;
class CP_SlotObject;

class CP_Signal
{
public:
										CP_Signal();
	virtual								~CP_Signal();

	virtual void						slot_disconnect( CP_SlotObject *inSlot ) = 0;

	// class data
protected:
};

class CP_SlotObject 
{
public:
										CP_SlotObject();
		virtual 						~CP_SlotObject();

		void 							disconnect_all();

		void 							signal_connect( CP_Signal *inSender );

		void 							signal_disconnect( CP_Signal *inSender );

private:
	typedef std::set<CP_Signal *> 		sender_set;
	typedef sender_set::const_iterator	const_iterator;
	
	// class data
protected:
	sender_set							fSenders;
};

class CP_Slot
{
public:
										CP_Slot();
										CP_Slot( CP_Slot *inSlot );
										CP_Slot( const CP_Slot& inSlot );
	virtual								~CP_Slot();

				void 					emit();

		virtual CP_SlotObject			*getdest() const;
				CP_Slot					*GetSlot();
		
				void					signalConnect( CP_Signal *inCP_Signal ) const;

	// class data
private:
	CP_Slot								*fSlot;
};

class CP_Slot0 : public CP_Slot
{
public:
										CP_Slot0();
										CP_Slot0( CP_Slot *inSlot );
										CP_Slot0( const CP_Slot0& inSlot );
	virtual								~CP_Slot0();
};

template<class P1>
class CP_Slot1 : public CP_Slot
{
public:
										CP_Slot1()									{ fSlot = NULL; }
										CP_Slot1( CP_Slot1<P1> *inSlot )			{ fSlot = inSlot; }
										CP_Slot1( const CP_Slot1<P1>& inSlot )		{ fSlot = inSlot.fSlot; }
	virtual								~CP_Slot1()									{}

	virtual	void						emit( P1 )									{}

				CP_Slot1<P1>			*GetSlot()									{ return( fSlot ); }

private:
	CP_Slot1<P1>						*fSlot;
};

template<class P1, class P2>
class CP_Slot2 : public CP_Slot
{
public:
										CP_Slot2()
											{ fSlot = NULL; }
										CP_Slot2( CP_Slot2<P1, P2> *inSlot )
											{ fSlot = inSlot; }
										CP_Slot2( const CP_Slot2<P1, P2>& inSlot )
											{ fSlot = inSlot.fSlot; }
	virtual								~CP_Slot2()									{}

	virtual	void						emit( P1, P2 )								{}

	CP_Slot2<P1, P2>					*GetSlot()									{ return( fSlot ); }

				void					SetNULLSlot()								{ fSlot = NULL; }

private:
	CP_Slot2<P1, P2>					*fSlot;
};

template<class P1, class P2, class P3>
class CP_Slot3 : public CP_Slot
{
public:
										CP_Slot3()
											{ fSlot = NULL; }
										CP_Slot3( CP_Slot3<P1, P2, P3> *inSlot )
											{ fSlot = inSlot; }
										CP_Slot3( const CP_Slot3<P1, P2, P3>& inSlot )
											{ fSlot = inSlot.fSlot; }
	virtual								~CP_Slot3()									{}

	virtual	void						emit( P1, P2, P3 )							{}

	CP_Slot3<P1, P2, P3>				*GetSlot()									{ return( fSlot ); }

				void					SetNULLSlot()								{ fSlot = NULL; }

private:	
	CP_Slot3<P1, P2, P3>				*fSlot;
};

template<class dest_type>
class CP_SlotConnection0 : public CP_Slot0
{
public:
										CP_SlotConnection0()
										{
											fPobject = NULL;
											fPmemfun = NULL;
										}

										CP_SlotConnection0( dest_type *pobject, void (dest_type::*pmemfun)())
										{
											fPobject = pobject;
											fPmemfun = pmemfun;
										}

	virtual								~CP_SlotConnection0()							{}

			void						signalConnect( CP_Signal *inCP_Signal )			{ fPobject->CP_Signal_connect( inCP_Signal ); };

	virtual void 						emit()											{ (fPobject->*fPmemfun)(); }

	virtual CP_SlotObject				*getdest() const								{ return( fPobject ); }

private:
	dest_type							*fPobject;
	void								(dest_type::* fPmemfun)();
};

template<class dest_type, class P1>
class CP_SlotConnection1 : public CP_Slot1<P1>
{
public:
										CP_SlotConnection1()
										{
											fPobject = NULL;
											fPmemfun = NULL;
										}

										CP_SlotConnection1( dest_type* pobject, void (dest_type::*pmemfun)(P1) )
										{
											fPobject = pobject;
											fPmemfun = pmemfun;
										}

										virtual	~CP_SlotConnection1()					{}

			void						signalConnect( CP_Signal *inCP_Signal )			{ fPobject->signal_connect( inCP_Signal ); };

	virtual void 						emit( P1 a1 )									{ (fPobject->*fPmemfun)(a1); }

	virtual CP_SlotObject				*getdest() const								{ return( fPobject ); }

private:
	dest_type							*fPobject;
	void 								(dest_type::* fPmemfun)(P1);
};

template<class dest_type, class P1, class P2>
class CP_SlotConnection2 : public CP_Slot2<P1, P2>
{
public:
										CP_SlotConnection2()
										{
											fPobject = NULL;
											fPmemfun = NULL;
										}

										CP_SlotConnection2( dest_type* pobject, void (dest_type::*pmemfun)(P1, P2) )
										{
											fPobject = pobject;
											fPmemfun = pmemfun;
										}

										virtual	~CP_SlotConnection2()					{}

			void						signalConnect( CP_Signal *inSignal )			{ fPobject->signal_connect( inSignal ); };

	virtual void 						emit( P1 a1, P2 a2 )							{ (fPobject->*fPmemfun)(a1, a2); }

	virtual CP_SlotObject				*getdest() const								{ return( fPobject ); }

private:
	dest_type							*fPobject;
	void 								(dest_type::* fPmemfun)(P1, P2);
};

template<class dest_type, class P1, class P2, class P3>
class CP_SlotConnection3 : public CP_Slot3<P1, P2, P3>
{
public:
										CP_SlotConnection3()
										{
											fPobject = NULL;
											fPmemfun = NULL;
										}

										CP_SlotConnection3( dest_type* pobject, void (dest_type::*pmemfun)(P1, P2, P3) )
										{
											fPobject = pobject;
											fPmemfun = pmemfun;
										}

										virtual	~CP_SlotConnection3()					{}

			void						signalConnect( CP_Signal *inSignal )			{ fPobject->signal_connect( inSignal ); };

	virtual void 						emit( P1 a1, P2 a2, P3 a3 )						{ (fPobject->*fPmemfun)(a1, a2, a3); }

	virtual CP_SlotObject				*getdest() const								{ return( fPobject ); }

private:
	dest_type							*fPobject;
	void 								(dest_type::* fPmemfun)(P1, P2, P3);
};

class CP_Signal0 : public CP_Signal
{
public:
	typedef std::list<CP_Slot *>																			connections_list;

										CP_Signal0();
	virtual								~CP_Signal0();

			void 						disconnect_all();

			void 						disconnect( CP_SlotObject *inSlotObject );

			void						connect( const CP_Slot& CP_Slot );
										
			void						emit() const;
			void						add( const CP_Slot& CP_Slot );

			void						slot_disconnect( CP_SlotObject *pCP_Slot );

protected:
	connections_list					fConnectedSlots;   
};

template<class P1>
class CP_Signal1 : public CP_Signal
{
public:
	typedef std::list<CP_Slot1<P1> *>															connections_list;
	typedef typename std::list<CP_Slot1<P1>*,std::allocator<CP_Slot1<P1> *> >::const_iterator	connections_list_const_iterator;
	typedef typename std::list<CP_Slot1<P1>*,std::allocator<CP_Slot1<P1> *> >::iterator			connections_list_iterator;

										CP_Signal1()								{}
	virtual								~CP_Signal1()								{ disconnect_all(); }

			void						disconnect_all()
											{
												connections_list_const_iterator		it = fConnectedSlots.begin();
												connections_list_const_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													CP_Slot1<P1>			*slot = *it;
													CP_Slot1<P1>			*slot1Slot = slot->GetSlot();
													
													if( slot1Slot != NULL )
													{
														slot1Slot->getdest()->signal_disconnect(this);
														
														delete slot1Slot;
													}
													
													delete *it;

													++it;
												}

												fConnectedSlots.erase( fConnectedSlots.begin(), fConnectedSlots.end() );
											}

		void 							disconnect( CP_SlotObject *inSlotObject )
												{
													connections_list_iterator 		it = fConnectedSlots.begin();
													connections_list_iterator 		itEnd = fConnectedSlots.end();

													while( it != itEnd )
													{
														if( (*it)->getdest() == inSlotObject )
														{
															delete *it;
															
															fConnectedSlots.erase( it );
															
															inSlotObject->signal_disconnect( this );
															
															return;
														}

														++it;
													}
												}

			void						connect( const CP_Slot1<P1>& inSlot )
										{
											add(inSlot);
											
											inSlot.signalConnect( this );
										}
										
	virtual	void						emit( P1 a1 )
											{
												connections_list_iterator		itNext;
												connections_list_iterator		it = fConnectedSlots.begin();
												connections_list_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													itNext = it;
													++itNext;

													CP_Slot1<P1>			*slot = *it;
													
													if( slot->GetSlot() != NULL )
														slot->GetSlot()->emit(a1);

													it = itNext;
												}
											}
											
			void						add( const CP_Slot1<P1>& inSlot )
											{
												CP_Slot1<P1>		*slot = new CP_Slot1<P1>( inSlot );
												
												fConnectedSlots.push_back( slot);
											}

			void						slot_disconnect( CP_SlotObject *inSlotObject )
											{
												connections_list_iterator		it = fConnectedSlots.begin();
												connections_list_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													connections_list_iterator	itNext = it;
													++itNext;

													if( (*it)->getdest() == inSlotObject )
														fConnectedSlots.erase(it);

													it = itNext;
												}
											}

protected:
	connections_list					fConnectedSlots;   
};

template<class P1, class P2>
class CP_Signal2 : public CP_Signal
{
public:
	typedef std::list<CP_Slot2<P1, P2> *>																connections_list;
	typedef typename std::list<CP_Slot2<P1, P2>*,std::allocator<CP_Slot2<P1, P2> *> >::const_iterator	connections_list_const_iterator;
	typedef typename std::list<CP_Slot2<P1, P2>*,std::allocator<CP_Slot2<P1, P2> *> >::iterator			connections_list_iterator;

										CP_Signal2()								{}
	virtual								~CP_Signal2()								{ disconnect_all(); }

			void						disconnect_all()
											{
												connections_list_const_iterator		it = fConnectedSlots.begin();
												connections_list_const_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													CP_Slot2<P1, P2>			*slot = *it;
													CP_Slot2<P1, P2>			*slot2Slot = slot->GetSlot();
													
													if( slot2Slot != NULL )
													{
														if( slot2Slot->getdest() != NULL )
															slot2Slot->getdest()->signal_disconnect(this);
														
														delete slot2Slot;
													}
													
													delete *it;

													++it;
												}

												fConnectedSlots.erase( fConnectedSlots.begin(), fConnectedSlots.end() );
											}

		void 							disconnect( CP_SlotObject *inSlotObject )
												{
													connections_list_iterator 		it = fConnectedSlots.begin();
													connections_list_iterator 		itEnd = fConnectedSlots.end();

													while( it != itEnd )
													{
														if( (*it)->getdest() == inSlotObject )
														{
															delete *it;
															
															fConnectedSlots.erase( it );
															
															inSlotObject->signal_disconnect( this );
															
															return;
														}

														++it;
													}
												}

			void						connect( const CP_Slot2<P1, P2>& inSlot )
										{
											add(inSlot);
											
											inSlot.signalConnect( this );
										}
										
	virtual	void						emit( P1 a1, P2 a2 )
											{
												connections_list_iterator		itNext;
												connections_list_iterator		it = fConnectedSlots.begin();
												connections_list_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													itNext = it;
													++itNext;

													CP_Slot2<P1, P2>			*slot = *it;
													
													if( slot->GetSlot() != NULL )
														slot->GetSlot()->emit(a1, a2);

													it = itNext;
												}
											}
											
			void						add( const CP_Slot2<P1, P2>& inSlot )
											{
												CP_Slot2<P1, P2>		*slot = new CP_Slot2<P1, P2>( inSlot );
												
												fConnectedSlots.push_back( slot);
											}

			void						slot_disconnect( CP_SlotObject *inSlotObject )
											{
												connections_list_iterator		it = fConnectedSlots.begin();
												connections_list_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													connections_list_iterator	itNext = it;
													++itNext;

													if( (*it)->getdest() == inSlotObject )
														fConnectedSlots.erase(it);

													it = itNext;
												}
											}

protected:
	connections_list					fConnectedSlots;   
};

template<class P1, class P2, class P3>
class CP_Signal3 : public CP_Signal
{
public:
	typedef std::list<CP_Slot3<P1, P2, P3> *>																	connections_list;
	typedef typename std::list<CP_Slot3<P1, P2, P3>*,std::allocator<CP_Slot3<P1, P2, P3> *> >::const_iterator	connections_list_const_iterator;
	typedef typename std::list<CP_Slot3<P1, P2, P3>*,std::allocator<CP_Slot3<P1, P2, P3> *> >::iterator			connections_list_iterator;

										CP_Signal3()								{}
	virtual								~CP_Signal3()								{ disconnect_all(); }

			void						disconnect_all()
											{
												connections_list_const_iterator		it = fConnectedSlots.begin();
												connections_list_const_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													CP_Slot3<P1, P2, P3>			*slot = *it;
													CP_Slot3<P1, P2, P3>			*slot3Slot = slot->GetSlot();
													
													if( slot3Slot != NULL )
													{
														slot3Slot->getdest()->signal_disconnect(this);
														
														delete slot3Slot;
													}
													
													delete *it;

													++it;
												}

												fConnectedSlots.erase( fConnectedSlots.begin(), fConnectedSlots.end() );
											}

		void 							disconnect( CP_SlotObject *inSlotObject )
												{
													connections_list_iterator 		it = fConnectedSlots.begin();
													connections_list_iterator 		itEnd = fConnectedSlots.end();

													while( it != itEnd )
													{
														if( (*it)->getdest() == inSlotObject )
														{
															delete *it;
															
															fConnectedSlots.erase( it );
															
															inSlotObject->signal_disconnect( this );
															
															return;
														}

														++it;
													}
												}

			void						connect( const CP_Slot3<P1, P2, P3>& inSlot )
										{
											add(inSlot);
											
											inSlot.signalConnect( this );
										}
										
	virtual	void						emit( P1 a1, P2 a2, P3 a3 )
											{
												connections_list_iterator		itNext;
												connections_list_iterator		it = fConnectedSlots.begin();
												connections_list_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													itNext = it;
													++itNext;

													CP_Slot3<P1, P2, P3>		*slot = *it;
													
													if( slot->GetSlot() != NULL )
														slot->GetSlot()->emit(a1, a2, a3);

													it = itNext;
												}
											}
											
			void						add( const CP_Slot3<P1, P2, P3>& inSlot )
											{
												CP_Slot3<P1, P2, P3>		*slot = new CP_Slot3<P1, P2, P3>( inSlot );
												
												fConnectedSlots.push_back( slot);
											}

			void						slot_disconnect( CP_SlotObject *inSlotObject )
											{
												connections_list_iterator		it = fConnectedSlots.begin();
												connections_list_iterator		itEnd = fConnectedSlots.end();

												while( it != itEnd )
												{
													connections_list_iterator	itNext = it;
													++itNext;

													if( (*it)->getdest() == inSlotObject )
														fConnectedSlots.erase(it);

													it = itNext;
												}
											}

protected:
	connections_list					fConnectedSlots;   
};

template<class dest_type>
CP_Slot0 slot(dest_type* pobject, void (dest_type::*pmemfun)())
{
	CP_SlotConnection0<dest_type>			*slotConn = new CP_SlotConnection0<dest_type>(pobject, pmemfun);
	
	return( CP_Slot0( slotConn ) );
}

template<class dest_type, class P1>
CP_Slot1<P1> slot(dest_type* pobject, void (dest_type::*pmemfun)(P1))
{
	CP_SlotConnection1<dest_type, P1>		*slotConn = new CP_SlotConnection1<dest_type, P1>(pobject, pmemfun);

	return( CP_Slot1<P1>( slotConn ) );
}

template<class dest_type, class P1, class P2>
CP_Slot2<P1, P2> slot(dest_type* pobject, void (dest_type::*pmemfun)(P1, P2))
{
	CP_SlotConnection2<dest_type, P1, P2>		*slotConn = new CP_SlotConnection2<dest_type, P1, P2>(pobject, pmemfun);

	return( CP_Slot2<P1, P2>( slotConn ) );
}

template<class dest_type, class P1, class P2, class P3>
CP_Slot3<P1, P2, P3> slot(dest_type* pobject, void (dest_type::*pmemfun)(P1, P2, P3))
{
	CP_SlotConnection3<dest_type, P1, P2, P3>		*slotConn = new CP_SlotConnection3<dest_type, P1, P2, P3>(pobject, pmemfun);

	return( CP_Slot3<P1, P2, P3>( slotConn ) );
}

}	//namespace CP_SigSlot

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SignalSlot

