//=========================================================================
//
//	FILE:				CP_Time.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class for dealing with times.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 145 $
//						Latest Checkout:  $Date: 2005-04-14 16:01:09 -0700 (Thu, 14 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_Time
#define _H_CP_Time
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_String.h"
#include "CP_Types.h"

	// Standard Library
#include <ctime>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Time
*	@brief		A class for dealing with times.
*	
*				The CP_Times class provides methods for performing math on times.
*
*				There are also methods to return the time in a specific formatted way.
*/
class CP_EXPORT CP_Time
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Time();
										CP_Time( CP_STD::time_t inTime );
	//! Destructor.
										~CP_Time();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the hour portion of the time.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		Hour portion of the time.
										*/
			SInt16						GetHour()											{ return( fHour ); }

										/*!
										*	Returns the minute portion of the time.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		Minute portion of the time.
										*/
			SInt16						GetMinute()											{ return( fMinute ); }

										/*!
										*	Returns the seconds portion of the time.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		Seconds portion of the time.
										*/
			SInt16						GetSecond()											{ return( fSecond ); }
	//@}

	//! @name Math
	//@{
			const CP_Time&				AddHours( SInt16 inHours );
			const CP_Time&				SubtractHours( SInt16 inHours );
			const CP_Time&				AddMinutes( SInt16 inMinutes );
			const CP_Time&				SubtractMinutes( SInt16 inMinutes );
			const CP_Time&				AddSeconds( SInt16 inSeconds );
			const CP_Time&				SubtractSeconds( SInt16 inSeconds );
	//@}

	//! @name Format
	//@{
			CP_String					Format( char *inFormat );
	//@}
			
	//! @name Setters
	//@{
			void						SetTimeNow();

										/*!
										*	Sets the hour portion of the time.
										*
										*	@param		inHour			The hour portion of the time.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetHour( SInt16 inHour )							{ fHour = inHour; }

										/*!
										*	Sets the minute portion of the time.
										*
										*	@param		inMinute		The minute portion of the time.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetMinute( SInt16 inMinute )						{ fMinute = inMinute; }

										/*!
										*	Sets the seconds portion of the time.
										*
										*	@param		inSecond			The seconds portion of the time.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetSecond( SInt16 inSecond )						{ fSecond = inSecond; }
	//@}

	// class data
protected:
	CP_STD::time_t						fTime;			/*! time_t representation of time.  */

	SInt16								fHour;			/*! Hour of the time. */
	SInt16								fMinute;		/*! Minute of the time. */
	SInt16								fSecond;		/*! Seconds of the time. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
										CP_Time(
												UInt32 inMacTime,
												bool inIsMacTime );
										CP_Time( LongDateRec inDateRec );
	//@}

	//! @name Getters
	//@{
			void						GetTime( LongDateRec& inOutDateRec );
	//@}

	//! @name Setters
	//@{
			void						SetTime( LongDateRec inDateRec );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
										CP_Time( SYSTEMTIME inWinTime );
	//@}

	//! @name Getters
	//@{
			void						GetTime( SYSTEMTIME& inOutWinTime );
	//@}

	//! @name Setters
	//@{
			void						SetTime( SYSTEMTIME inWinTime );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_Time
