//=========================================================================
//
//	FILE:				CP_Utility.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Various utility routines used in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 170 $
//						Latest Checkout:  $Date: 2005-05-12 23:53:08 -0700 (Thu, 12 May 2005) $
//
//=========================================================================
#ifndef _H_CP_Utility
#define _H_CP_Utility
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_OSTypes.h"
#include "CP_OSStatus.h"
#include "CP_SmartPtr.h"
#include "CP_String.h"
#include "CP_Types.h"

	// Standard Library
#include <algorithm>
#include <exception>

CPLAT_Begin_Namespace_CPLAT

namespace CP_Utility
{
	//=========================================================================
	// Bit manipulation
	//=========================================================================
	void				CP_FlipFlopFlag(
								SInt16 *inFlag,
								SInt16 inStyle );

	void				CP_FormatHexDigit(
								UInt8 inByte,
								char *outBuffer );
	UInt8 				CP_HexDigitToByte( const char *inBuffer );
	inline char 		CP_HexDigitToChar( const char *inBuffer )			{ return static_cast<char>(CP_HexDigitToByte(inBuffer)); }
	void 				CP_LongToHex(
								const SInt32 inLong,
								const bool inShowAs24Bits,
								char *inOutBuffer,
								SInt16 inOutBufferLength );

	//=========================================================================
	// Memory handling
	//=========================================================================
	void				CP_CopyMemory(
								const void *srcPtr,
								void *dstPtr,
								const UInt32 inBytesToCopy );
	void				CP_SetMemory(
								const void *srcPtr,
								const UInt32 inNumBytes,
								const unsigned char value );
	bool				CP_AreBlocksEqual(
								const void *inLeftBlock,
								const void	*inRightBlock,
								UInt32 inLength );

	void 				CP_FillWithGarbage(
								void *ptr,
								const UInt32 inLength );

	//=========================================================================
	// Misc
	//=========================================================================
	void				CP_Beep( bool simple );
	bool 				CP_IsKeyDown( SInt16 inWhichKey );

	//=========================================================================
	// String handling
	//=========================================================================
	UInt32				CP_CStrLength( const char *inString );
	UInt32				CP_WStrLength( const CPUniChar *inStr );

							// conversions
							// char
	void				CP_CToPString(
								const char *inString,
								unsigned char *outString );
	void				CP_PToCString(
								const CP_String& inString,
								unsigned char *outString,
								UInt16 inMaxLength );
	void				CP_PToCString(
								const unsigned char *inString,
								char *outString,
								UInt16 outMaxStringLength );

	void				CP_CopyCStrings(
								const char *inString,
								char *inDestString,
								UInt32 maxDestStringLength = kCP_PStyleString_MaxLength );
	void 				CP_AppendCString(
								const char *srcStr,
								char *destStr );
	void 				CP_AppendPString(
								const unsigned char *srcStr,
								unsigned char *destStr );

	CP_String			CP_PToString( const unsigned char *inString );
	void				CP_StringToPString(
								const CP_String& inString,
								unsigned char *outString,
								UInt16 inMaxLength = kCP_PStyleString_MaxLength );

	void				CP_StringToCString(
								const CP_String& inString,
								char *outString,
								UInt32 inMaxLength );
	CP_String			CP_CStringToString( const char *inString );

	CP_String			CP_StringToUpper( const CP_String& inString );
	CP_String			CP_StringToLower( const CP_String& inString );

							// types to string
	CP_String			CP_IntToString( int inInteger );
	CP_String			CP_SInt16ToString( const SInt16 inSInt16 );
	CP_String			CP_UInt16ToString( const UInt16 inUInt16 );
	CP_String			CP_SInt32ToString( SInt32 inSInt32 );
	CP_String			CP_UInt32ToString( const UInt32 inUInt32 );
	CP_String			CP_SInt64ToString( const SInt64 inSInt64 );
	CP_String			CP_UInt64ToString( const UInt64 inUInt64 );
	CP_String 			CP_FloatToString( const float inFloat );
	CP_String	 		CP_FourCharCodeToString( const FourCharCode inCode );
	CP_String			CP_BytesToHexString( const void *inBytes, SInt32 inNumBytes, SInt16 inWordLength = 0, SInt16 inLeadingSpaces = 0, SInt32 inBreakLineAfterChars = -1 );
	CP_String			CP_VersionToString( const UInt32 inVersion );

							// string to types
	SInt16				CP_StringToSInt16( const CP_String& inString );
	UInt16 				CP_StringToUInt16( const CP_String& inString );
	SInt32				CP_StringToSInt32( const CP_String& inStringData );
	UInt32				CP_StringToUInt32( const CP_String& inStringData );
	double				CP_StringToDouble( const CP_String& inString );
	float				CP_StringToFloat( const CP_String& inString );
	FourCharCode 		CP_StringToFourCharCode( const CP_String& inStringData );
	UInt32				CP_StringToVersion( const CP_String& inString);	

	#if TARGET_OS_MAC
		DescType 		CP_StringToDescType( const CP_String& inStringData );
	#endif

	// ===================================================================================
	//	STL Helpers
	// ===================================================================================
	template <class Collection, class T>
	bool 		member( const Collection& collection, const T& value );

	template <class Collection, class T>
	inline bool member( const Collection& collection, const T& value )
	{
		return( std::find( collection.begin(), collection.end(), value ) != collection.end() );
	}

	#if TARGET_OS_LINUX
/*
		template <typename KEY, typename VALUE>
		struct CP_MapKey : public unary_function< std::pair<const KEY, VALUE>, KEY>
		{
			const KEY& operator () ( const std::pair<const KEY, VALUE>& inElement ) const
			{
				return( inElement.first );
			}
		};

		template <typename KEY, typename VALUE>
		struct CP_MapValue : public unary_function< std::pair<const KEY, VALUE>, VALUE>
		{
			const VALUE& operator () (const std::pair<const KEY, VALUE>& inElement) const
			{
				return( inElement.second );
			}
		};
*/
	#else
		#if ! TARGET_CC_MSVC
		template <typename KEY, typename VALUE>
		struct CP_MapKey : public std::unary_function< std::pair<const KEY, VALUE>, KEY>
		{
			const KEY& operator () ( const std::pair<const KEY, VALUE>& inElement ) const
			{
				return( inElement.first );
			}
		};

		template <typename KEY, typename VALUE>
		struct CP_MapValue : public std::unary_function< std::pair<const KEY, VALUE>, VALUE>
		{
			const VALUE& operator () (const std::pair<const KEY, VALUE>& inElement) const
			{
				return( inElement.second );
			}
		};
		#endif	// ! TARGET_CC_MSVC
	#endif	// TARGET_OS_LINUX

	#if TARGET_OS_MAC

	// ===================================================================================
	//	Mac Specific
	// ===================================================================================

	OSStatus			CP_AEGetDescData(
								AEDesc& inDesc,
								DescType& inOutTypeCode,
								void *inDataBuffer,
								ByteCount inMaximumSize,
								ByteCount& inOutActualSize );

	#endif	// TARGET_OS_MAC

	void CP_RaiseSafeCastException(const char* whichCast);
	
}	// namespace CP_Utility

template <typename RESULT, typename SOURCE>
inline RESULT safe_cast(SOURCE *inSrc)
{
	RESULT	result = dynamic_cast<RESULT>(inSrc);
	if ( result == NULL  &&  inSrc != NULL ) {
#if TARGET_CC_GNUC
		CP_Utility::CP_RaiseSafeCastException(__PRETTY_FUNCTION__);
#else
		CP_Utility::CP_RaiseSafeCastException(NULL);
#endif
	}
	return result;
}
template <typename RESULT, typename SOURCE>
inline RESULT safe_cast(const CP_SmartPtr<SOURCE>& inSrc)
{
	RESULT	result = dynamic_cast<RESULT>(inSrc.Get());
	if ( result == NULL  &&  inSrc.Get() != NULL ) {
#if TARGET_CC_GNUC
		CP_Utility::CP_RaiseSafeCastException(__PRETTY_FUNCTION__);
#else
		CP_Utility::CP_RaiseSafeCastException(NULL);
#endif
	}
	return result;
}
template <typename RESULT, typename SOURCE>
inline RESULT safe_cast(const CP_SmartRef<SOURCE>& inSrc)
{
	RESULT	result = dynamic_cast<RESULT>(inSrc.Get());
	if ( result == NULL  &&  inSrc.Get() != NULL ) {
#if TARGET_CC_GNUC
		CP_Utility::CP_RaiseSafeCastException(__PRETTY_FUNCTION__);
#else
		CP_Utility::CP_RaiseSafeCastException(NULL);
#endif
	}
	return result;
}

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Utility

