//=========================================================================
//
//	FILE:				CP_Document_BaseManager.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Singleton class that handle a list of CP_Document_Base objects.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 218 $
//						Latest Checkout:  $Date: 2005-06-22 23:43:56 -0700 (Wed, 22 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Document_BaseManager
#define _H_CP_Document_BaseManager
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_AskReviewDialog.h"
#include "CP_Document_Base.h"
#include "CP_NoCopy.h"

	// Standard library
#include <list>

CPLAT_Begin_Namespace_CPLAT

	// typedefs
/*!	\fn typedef CP_Document_BasesList
 *	A std::list object that holds a list of CP_Document_Bases. 
 */
#ifndef TARGET_OS_LINUX
	typedef std::list<CP_Document_Base *, std::allocator<CP_Document_Base *> >						CP_Documents_List;
	typedef std::list<CP_Document_Base *, std::allocator<CP_Document_Base *> >::iterator			CP_Documents_Iterator;
	typedef std::list<CP_Document_Base *, std::allocator<CP_Document_Base *> >::reverse_iterator	CP_Documents_ReverseIterator;
#else
	typedef std::list<CP_Document_Base *>															CP_Documents_List;
	typedef std::list<CP_Document_Base *>::iterator													CP_Documents_Iterator;
	typedef std::list<CP_Document_Base *>::reverse_iterator											CP_Documents_ReverseIterator;
#endif

/*!
*	@class		CP_DocumentManager
*	@brief		Singleton class to manage all open document objects.
*/
class CP_EXPORT CP_DocumentManager : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DocumentManager();
	//! Destructor.
	virtual								~CP_DocumentManager();
	//@}
			
	//! @name Instance
	//@{
	static	CP_DocumentManager			*Instance();
	//@}

	//! @name Commands
	//@{
			bool						CloseAll();
			bool						CloseAllNoSave();
	//@}
			
	//! @name Document List
	//@{
			UInt32						Count();
			void						Append( CP_Document_Base *inDocument );
			void						Remove( CP_Document_Base *inDocument );

			CP_Document_Base			*GetNthDocument( SInt32 inIndex );

										/*!
										*	Returns the list of documents managed by this class.
										*
										*	@exception	none
										*	
										*	@return		CP_Documents_List	The list of documents managed by this class.
										*/
			CP_Documents_List			*GetDocumentsList()							{ return( fDocuments ); }	
	//@}

protected:

	// class data
protected:
	static CP_DocumentManager			*fsInstance;

	CP_Documents_List					*fDocuments;		/*! List of all documents. */
	
	bool								fCloseAllInProgress;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
			void						CloseEach();
			
protected:
			CP_AskReviewDialog 			*GetAskReviewDialog( UInt32 inNumberDocuments );

#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Document_BaseManager

