//=========================================================================
//
//	FILE:				CP_IBClassToCPXML.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class and set of routines to convert XML contained
//						in an Interface Builder XIB xml file to that needed
//						by the CPLAT framework.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 145 $
//						Latest Checkout:  $Date: 2005-04-14 16:01:09 -0700 (Thu, 14 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_IBClassToCPXML
#define _H_CP_IBClassToCPXML
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_XMLDocument.h"
#include "CP_XMLWriter.h"

CPLAT_Begin_Namespace_CPLAT

typedef void (*IBClassToXMLConverterCallback)( CP_XMLDocNode *inParentNode, const CP_String& inIBClassName, CP_XMLWriter *inXMLWriter, CP_XMLWriter_Element *inParentElement, CP_Rect& inOutBounds, bool& inOutInPortBounds );

#if TARGET_OS_LINUX
	typedef std::map<CP_String, IBClassToXMLConverterCallback, std::less<CP_String> >					CP_IBClassToXMLWriter_Map;
	typedef std::map<CP_String, IBClassToXMLConverterCallback, std::less<CP_String> >::iterator			CP_IBClassToXMLWriter_Iterator;

	typedef std::pair<const CP_String, IBClassToXMLConverterCallback>									CP_IBClassToXMLWriter_Entry;
#else
		// IBClassName to converter mapping
	typedef std::map<CP_String, IBClassToXMLConverterCallback, std::less<CP_String> >					CP_IBClassToXMLWriter_Map;
	typedef std::map<CP_String, IBClassToXMLConverterCallback, std::less<CP_String> >::iterator			CP_IBClassToXMLWriter_Iterator;

	typedef std::pair<const CP_String, IBClassToXMLConverterCallback>									CP_IBClassToXMLWriter_Entry;

		// Custom control id to CPLAT class name mapping
	typedef std::map<UInt32, const CP_String, std::less<UInt32> >										CP_CustomControlToCPLATClass_Map;
	typedef std::map<UInt32, const CP_String, std::less<UInt32> >::iterator								CP_CustomControlToCPLATClass_Iterator;

	typedef std::pair<const UInt32, const CP_String>													CP_CustomControlToCPLATClass_Entry;
#endif

class CP_EXPORT CP_IBClassToCPXML : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_IBClassToCPXML();
	//! Destructor.
	virtual								~CP_IBClassToCPXML() throw();
	//@}
			
	//! @name Instance
	//@{
	static	CP_IBClassToCPXML			*Instance();
	//@}

	//! @name IBClassToXMLConversion
	//@{
			void						RegisterIBClassToXMLConverter(
												char *inIBClassName,
												IBClassToXMLConverterCallback inCallback );
			IBClassToXMLConverterCallback	FindXMLConverterFromIBClassName( CP_String& inIBClassName );
	//@}

	//! @name CustomControlToCplat
	//@{
			void						RegisterCustomControlToCPLATClassName(
												UInt32 inCustomControlID,
												const CP_String& inCPLATClassName );
			CP_String					FindCPLATClassNameFromCustomControlID( UInt32 inCustomControlID );
	//@}

	//! @name Output
	//@{
			CP_FileSystemSpec			GetOutputDirectorySpec()										{ return( fOutputDirectorySpec ); }
			void						SetOutputDirectorySpec( const CP_FileSystemSpec& inSpec )		{ fOutputDirectorySpec = inSpec; }
	//@}

protected:

private:

	// class data
protected:
	static CP_IBClassToCPXML			*fsInstance;						/*! Single instance of this class. */

	CP_IBClassToXMLWriter_Map			fIBClasstoXMLArray;					/*! Maps a class name in IB to corresponding CPLAT class.
																				This map is filled in through users registering the
																				mapping. */
	
	CP_FileSystemSpec					fOutputDirectorySpec;				/*! Location to write the generated XML. */
			
		// Custom controls
	CP_CustomControlToCPLATClass_Map	fCustomControlIDToCPLATClassName;	/*! Mapping between an IB custom control and CPLAT classes that
																				do not correspond directly to an IB control. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#define CP_RegisterIBClassConverter( inClassName, inConverterFunction )		CPLAT::CP_IBClassToCPXML::Instance()->RegisterIBClassToXMLConverter( inClassName, inConverterFunction )
#define CP_RegisterCustomControlID( inCustomID, inClassName )				CPLAT::CP_IBClassToCPXML::Instance()->RegisterCustomControlToCPLATClassName( inCustomID, inClassName )

void						RegisterKnownIBClasses();

void						WriteStandardControlFields(
									CP_XMLDocNode *inParentNode,
									CP_XMLWriter_Element *inWidgetElement,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );

	// Converters
void						IBClassWindowToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );

void						IBClassBevelButtonToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassChasingArrowsToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassCheckBoxToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassClockDateToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassComboBoxToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassDisclosureTriangleToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassEditFieldToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassGroupBoxToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassIconControlToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassImageWellToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassLittleArrowsToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassPictureControlToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassProgressToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassPopupArrowToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassPopupMenuToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassPushButtonToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassRadioButtonToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassRadioGroupToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassRelevanceBarToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassRoundButtonToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassSeparatorToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassScrollBarToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassSliderToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassStaticTextToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassTabViewToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void 						WriteTabItems(
									CP_XMLDocNode *inParentNode,
									CP_XMLWriter_Element *inWidgetElement,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement );
void 						WriteTabItem(
									CP_XMLDocNode *inParentNode,
									CP_XMLWriter_Element *inWidgetElement,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement );
void						HandleTabSubViewSection(
									CP_XMLDocNode *inSubviewsParentNode,
									CP_Rect& inTabViewBounds );

CP_String					AskUserForViewFileName(
									const CP_String& inDefaultName,
									SInt16 inTabNumber );
void 						HandleDismiss(
									CP_DataValueHolder& inValue,
									OSStatus& inOutStatus );

void						IBClassTextViewToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassUserPaneViewToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						IBClassCustomControlToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );

	// Custom Classes
void						CPCustomControl_LittleArrowsBuddyToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_OpenGLToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_PlacardToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_QuickTimeMovieViewToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_SelectColorButtonToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_ScrollableViewToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_ScrollerToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_SplitterToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_TableViewToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_TVDataBrowserToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_ToolbarToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_ViewToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_WindowHeaderToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );
void						CPCustomControl_WindowListHeaderToXML(
									CP_XMLDocNode *inParentNode,
									const CP_String& inIBClassName,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement,
									CP_Rect& inOutBounds,
									bool& inOutInPortBounds );

	// Menus
CP_XMLDocNode 				*FindOptionalMenuItems(
									CP_XMLDocNode *inParentNode );
void						WriteMenu(
									CP_XMLDocNode *inParentNode,
									CP_XMLWriter_Element *inWidgetElement,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement );
void						WriteMenuItems(
									CP_XMLDocNode *inParentNode,
									CP_XMLWriter_Element *inWidgetElement,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement );
void						WriteMenuItem(
									CP_XMLDocNode *inParentNode,
									CP_XMLWriter_Element *inWidgetElement,
									CP_XMLWriter *inXMLWriter,
									CP_XMLWriter_Element *inParentElement );

	// Misc
bool						IBHasDictionaryNode( CP_XMLDocNode *inParentNode );
bool 						IBHasCPWNFrameParameter( CP_XMLDocNode *inDictionaryNode );

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_IBClassToCPXML
