//=========================================================================
//
//	FILE:				CP_EventHandler_Imp_Gtk.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation class for the CP_EventHandler when
//						using MacOS Carbon Events.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_EventHandler_Imp_Gtk
#define _H_CP_EventHandler_Imp_Gtk
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_LINUX

	// CPLAT_Framework
#include "CP_Event.h"
#include "CP_New.h"
#include "CP_NoCopy.h"
#include "CP_OSStatus.h"
#include "CP_SignalSlot_Defines.h"
#include "CP_Types.h"

	// Standard Library
#include <map>
#if TARGET_CC_METROWERKS
	#include <hash_map>
#endif

	// Wonderlib
#include "SignalSlot.h"

CPLAT_Begin_Namespace_CPLAT

struct CP_S_LinuxEventSpec
{
	UInt32				eventClass;
	UInt32				eventKind;
};

struct Less : public std::binary_function<CP_S_LinuxEventSpec, CP_S_LinuxEventSpec, bool>
{
	bool		operator()(const CP_S_LinuxEventSpec& x, const CP_S_LinuxEventSpec& y) const
					{
						return( (x.eventClass < y.eventClass) || 
							((x.eventClass == y.eventClass) && (x.eventKind < y.eventKind)));
					}
};

struct Hash : public std::unary_function<CP_S_LinuxEventSpec, size_t>
{
	size_t		operator()(const CP_S_LinuxEventSpec& key) const
					{
						return (key.eventClass | key.eventKind);
					}
};

struct Equal : public std::binary_function<CP_S_LinuxEventSpec, CP_S_LinuxEventSpec, bool>
{
	bool		operator()(const CP_S_LinuxEventSpec& x, const CP_S_LinuxEventSpec& y) const
					{
						return( (x.eventClass == y.eventClass) && (x.eventKind == y.eventKind) );
					}
};

/*! CP_EventHandlerMap
*
*	Holds a mapping between an CP_S_LinuxEventSpec and the corresponding CP_EventSignal to handle the event.
*/
#if TARGET_CC_METROWERKS
		// prefer hash_map if available for a little more speed
	typedef Metrowerks::hash_map<CP_S_LinuxEventSpec, CP_EventSignal *, Hash, Equal >				CP_EventHandler_Map;
	typedef Metrowerks::hash_map<CP_S_LinuxEventSpec, CP_EventSignal *, Hash, Equal >::iterator		CP_EventHandler_Iterator;
#else
	typedef std::map<CP_S_LinuxEventSpec, CP_EventSignal *, Less >									CP_EventHandler_Map;
	typedef std::map<CP_S_LinuxEventSpec, CP_EventSignal *, Less >::iterator						CP_EventHandler_Iterator;
#endif

typedef std::pair<const CP_S_LinuxEventSpec, CP_EventSignal *>										CP_EventHandler_Entry;

/*!
*	@class		CP_EventHandler_Imp
*	@brief		Implementation class for the CP_EventHandler when using MacOS Carbon Events.
*
*	@ingroup	Events
*/
class CP_EXPORT CP_EventHandler_Imp : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_EventHandler_Imp();
	//! Destructor.
	virtual						~CP_EventHandler_Imp();
	//@}
			
	//! @name Events
	//@{
			void				Add(
										UInt32 inClass, 
										UInt32 inKind, 
										CP_EventSignal *inEventSignal,
										OSWindow inWindow = NULL );

			void				Remove(
										UInt32 inEventClass,
										UInt32 inEventKind );

			bool				HasHandler(
										UInt32 inEventClass,
										UInt32 inEventKind );

			CP_EventSignal		*FindHandler(
										UInt32 inEventClass,
										UInt32 inEventKind );
	//@}

protected:

private:

	// class data
protected:
	CP_EventHandler_Map			fEventHandlers;		/*! Map that contains all of the registered handlers. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_LINUX

#endif	// _H_CP_EventHandler_Imp_Gtk

