//=========================================================================
//
//	FILE:				CP_Graphics_Imp_Quartz_defs.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Defines for using Quartz functions in a CFM app.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 228 $
//						Latest Checkout:  $Date: 2005-07-04 16:35:04 -0700 (Mon, 04 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Graphics_Imp_Quartz_defs
#define _H_CP_Graphics_Imp_Quartz_defs
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_API_MAC_CARBON || defined(__CF_USE_FRAMEWORK_INCLUDES__)

	// CPLAT_Framework
#include "CP_Graphics_Imp.h"

CPLAT_Begin_Namespace_CPLAT

	// typedefs to call Quartz Mach functions from Carbon
typedef void (*CGContextSetAlphaFuncPtr)(CGContextRef c, float inAlpha);
typedef void (*CGContextSetLineWidthFuncPtr)(CGContextRef c, float lineWidth );

	// Graphics state
typedef void (*CGContextSaveGStateFuncPtr)(CGContextRef c);
typedef void (*CGContextRestoreGStateFuncPtr)(CGContextRef c);
typedef void (*CGContextSetLineJoinFuncPtr)(CGContextRef c, CGLineJoin join);
typedef void (*CGContextSetLineCapFuncPtr)(CGContextRef c, CGLineCap cap);
typedef void (*CGContextSetRGBStrokeColorFuncPtr)(CGContextRef c, float red, float green, float blue, float alpha );
typedef void (*CGContextSetRGBFillColorFuncPtr)(CGContextRef c, float red, float green, float blue, float alpha );

typedef void (*CGContextScaleCTMFuncPtr)(CGContextRef c, float sx, float sy );
typedef void (*CGContextTranslateCTMFuncPtr)(CGContextRef c, float tx, float ty );
typedef void (*CGContextRotateCTMFuncPtr)(CGContextRef c, float angle );

	// Bezier
typedef void (*CGContextAddCurveToPointFuncPtr)(CGContextRef c, float cp1x, float cp1y, float cp2x, float cp2y, float x, float y );
typedef void (*CGContextAddQuadCurveToPointFuncPtr)(CGContextRef c, float cp1x, float cp1y, float x, float y );

	// ColorSpace
typedef CGColorSpaceRef (*CGColorSpaceCreateDeviceRGBFuncPtr)();
typedef void (*CGColorSpaceReleaseFuncPtr)(CGColorSpaceRef cs );
	
	// Lines
typedef void (*CGContextMoveToPointFuncPtr)(CGContextRef c, float x, float y );
typedef void (*CGContextAddLineToPointFuncPtr)(CGContextRef c, float x, float y );
	
	// Images
typedef struct
{
	size_t				fWidth;
	size_t				fHeight;
	size_t				fBitsPerComponent;
	size_t				fBitsPerPixel;
	size_t				fBytesPerRow;
	size_t				fSize;
	CGImageAlphaInfo	fAI;
	CGColorSpaceRef		fCS;
	unsigned char		*fData;
	CMProfileRef		fProf;
} CP_S_QuartzBitmapInfo;

typedef void (*CGDataProviderReleaseFuncPtr)(CGDataProviderRef provider);
typedef CGDataProviderRef (*CGDataProviderCreateWithDataFuncPtr)(void *info, const void * data, size_t size, void (*releaseData)(void *info, const void *data, size_t size) );
typedef void (*CGImageReleaseFuncPtr)(CGImageRef image);
typedef void (*CGContextDrawImageFuncPtr)(CGContextRef ctx, CGRect rect, CGImageRef image );
#if TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1040
	typedef CGImageRef (*CGImageCreateFuncPtr)(size_t width, size_t height, size_t bitsPerComponent, size_t bitsPerPixel, size_t bytesPerRow,
		CGColorSpaceRef colorspace, CGBitmapInfo bitmapInfo, CGDataProviderRef provider, const float decode[], bool shouldInterpolate,
		CGColorRenderingIntent intent);
#else
	typedef CGImageRef (*CGImageCreateFuncPtr)(size_t width, size_t height, size_t bitsPerComponent,
		size_t bitsPerPixel, size_t bytesPerRow, CGColorSpaceRef colorspace, CGImageAlphaInfo alphaInfo,
		CGDataProviderRef provider, const float decode[], bool shouldInterpolate,
		CGColorRenderingIntent intent);
#endif

	// Paths
typedef void (*CGContextBeginPathFuncPtr)(CGContextRef c );
typedef void (*CGContextClosePathFuncPtr)(CGContextRef c );
typedef void (*CGContextStrokePathFuncPtr)(CGContextRef c );
typedef void (*CGContextFillPathFuncPtr)(CGContextRef c );
typedef void (*CGContextDrawPathFuncPtr)(CGContextRef c, CGPathDrawingMode mode );
typedef void (*CGContextAddRectFuncPtr)(CGContextRef c, CGRect rect);
typedef void (*CGContextAddArcFuncPtr)(CGContextRef c, float x, float y, float radius, float startAngle, float endAngle, int clockwise );	
typedef void (*CGContextAddArcToPointFuncPtr)(CGContextRef c, float x1, float y1, float x2, float y2, float radius );	
typedef void (*CGContextEOFillPathFuncPtr)(CGContextRef c );
typedef void (*CGContextClipFuncPtr)(CGContextRef c );

	// Rects
typedef void (*CGContextFillRectFuncPtr)(CGContextRef c, CGRect rect );
typedef void (*CGContextStrokeRectFuncPtr)(CGContextRef c, CGRect rect );

	// Text
typedef void (*CGContextShowTextAtPointFuncPtr)(CGContextRef ctx, float x, float y, const char *cstring, size_t length );
typedef void (*CGContextSelectFontFuncPtr)(CGContextRef ctx, const char * name, float size, CGTextEncoding textEncoding );

typedef CGAffineTransform (*CGAffineTransformScaleFuncPtr)(CGAffineTransform t, float sx, float sy);
typedef void (*CGContextSetTextMatrixFuncPtr)(CGContextRef c, CGAffineTransform t);
	
CPLAT_End_Namespace_CPLAT

#endif	// defined(TARGET_API_MAC_CARBON) || __CF_USE_FRAMEWORK_INCLUDES__

#endif	// _H_CP_Graphics_Imp_Quartz_defs
