//=========================================================================
//
//	FILE:				CP_Nib.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle loading and management of MacOS Nibs.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 267 $
//						Latest Checkout:  $Date: 2005-08-31 19:33:20 -0700 (Wed, 31 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Nib
#define _H_CP_Nib
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_CFBundle.h"
#include "CP_Menu.h"
#include "CP_NoCopy.h"
#include "CP_Window.h"
#include "CP_XMLDocument.h"

CPLAT_Begin_Namespace_CPLAT

typedef std::map<CPLAT::CP_String, CPLAT::CP_String, std::less<CPLAT::CP_String> >				CP_NibNameToRefRun_Map;
typedef std::map<CPLAT::CP_String, CPLAT::CP_String, std::less<CPLAT::CP_String> >::iterator	CP_NibNameToRefRun_Iterator;

typedef std::pair<const CPLAT::CP_String, const CPLAT::CP_String>								CP_NibNameToRefRun_Entry;

/*!
*	\class		CP_Nib
*	\brief		A class to handle loading and management of MacOS Nibs.
*/
class CP_EXPORT CP_Nib : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Nib();
	//! Destructor.
	virtual								~CP_Nib() throw();
	//@}
	
	//! @name Menus
	//@{
			CP_Menu						*LoadMenu(
												CP_CFBundle& inBundle,
												const CP_String& inNibFileName,
												const CP_String& inMenuName  );
	//@}

	//! @name Windows
	//@{
			CP_Window					*LoadWindow(
												CP_CFBundle& inBundle,
												const CP_String& inNibFileName,
												const CP_String& inWindowName );
	//@}
					
protected:
	//! @name Misc
	//@{
			CP_Window					*ConvertWindowInNib(
												CP_FileSystemSpec& inNibSpec,
												const CP_String& inWindowName );
			void						BuildNibNameMap( CP_XMLDocNode *inRootNode );
			CP_Window					*FindIBCarbonWindowNode(
												CP_XMLDocNode *inParentNode,
												const CP_String& inWindowName );
			CP_String					FindWindowRefFromRefID( CP_String& inRefID );
			void						CreateXMLWriter(
												CP_XMLWriter_Element **inOutRootElement,
												CPLAT::CP_XMLWriter_Element **inOutParentElement );
	//@}

	// class data
protected:
	IBNibRef							fNibRef;

	CP_NibNameToRefRun_Map				fNibNameToRefMap;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_Nib

