//=========================================================================
//
//	FILE:				CP_TV_DataBrowser.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base view class for displaying data in a table format.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 37 $
//						Latest Checkout:  $Date: 2005-01-01 12:08:06 -0800 (Sat, 01 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_TV_DataBrowser
#define _H_CP_TV_DataBrowser
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_InPlaceEditField.h"
#include "CP_OSTypes.h"
#include "CP_TableView.h"
#include "CP_TV_DataBrowser_Defines.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// forward class declaration
class CP_TV_DataBrowserRow;

typedef std::vector<CP_TV_DataBrowserRow *>										CP_Rows_Array;
typedef std::vector<CP_TV_DataBrowserRow *>::iterator							CP_Rows_Iterator;
typedef std::vector<CP_TV_DataBrowserRow *>::reverse_iterator					CP_Rows_ReverseIterator;

class CP_EXPORT CP_TV_DataBrowser : public CP_TableView
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TV_DataBrowser();
										CP_TV_DataBrowser(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												CP_S_TV_DB_Properties& inProperties );
	//@}
	//! Destructor.
	virtual								~CP_TV_DataBrowser() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Columns
	//@{
			void						SetColumnContentType(
												SInt32 inColumn,
												FourCharCode inContentType );
			CP_S_TV_DB_ColumnParams		GetColumnParams( SInt32 inColumn );
			void						SetColumnParams(
												SInt32 inColumn,
												CP_S_TV_DB_ColumnParams& inParams );

			void						SetColumnWidth(
												SInt32 inRow,
												SInt32 inColumn,
												SInt32 inColumnWidth );
	//@}

	//! @name ContentHandlers
	//@{
			void						RegisterHandler( CP_S_TV_DB_ContentHandlerInfo& inHandlerInfo );
			void						RemoveHandler( FourCharCode inContentType );
			CP_TV_DataBrowser_CH_Base 	*FindHandler( FourCharCode inContentType );
			CP_TV_DataBrowser_CH_Base 	*FindHandlerForColumn( SInt32 inColumn );

			CP_TV_DataBrowser_CH_Base	*GetBackgroundContentHandler()				{ return( fBackgroundContentHandler ); }
			void						SetBackgroundContentHandler( CP_TV_DataBrowser_CH_Base *inHandler );

			void						HandleContentHandlerEditExit( const CP_DataValueHolder& inValue );

			void						AdvanceContentHandlerEdit( CP_S_TV_DB_ParamBlock inParamBlock );
	//@}

	//! @name Misc
	//@{
			void						RecalcRowBounds( CP_TV_DataBrowserRow *inStartRow );
	
			CP_InPlaceEditField			*GetInPlaceEditField()									{ return( fInPlaceEditField ); }
			void						SetInPlaceEditField( CP_InPlaceEditField *inField )		{ fInPlaceEditField = inField; }
	//@}

	//! @name Rows
	//@{
			void						AddRootRow( CP_TV_DataBrowserRow *inRow );

			CP_TV_DataBrowserRow 		*GetRow( SInt32 inIndex );
			CP_TV_DataBrowserRow 		*FindRow( UInt32 inRowID );
			SInt32						GetRowIndex( CP_TV_DataBrowserRow *inDataBrowserRow );
			SInt32						GetNumberRows();
			
			void						SetRowHeight(
												SInt32 inRow,
												SInt32 inRowHeight );

			void						RemoveAllRows();
	//@}

	//! @name Properties
	//@{
			CP_S_TV_DB_Properties		GetProperties()									{ return( fProperties ); }
	//@}

	//! @name Rows
	//@{
			void						ExpandRow( CP_DataValueHolder& inDataValue );
	//@}

	//! @name Signal / Slots
	//@{
			void						SendClickedSignal( CP_S_TV_DB_ParamBlock& inParamBlock );
	//@}

	//! @name Sorting
	//@{
	virtual	bool						CompareRows( CP_S_TV_DB_SortInfo inSortInfo );
	virtual	void						Sort();
	//@}

protected:
	//! @name Drawomg
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

private:
	//! @name Initialization
	//@{
			void						Init( CP_S_TV_DB_Properties& inProperties );
	//@}
			
	//! @name Misc
	//@{
			void						RefreshRowRange(
												SInt32 inStartRow,
												SInt32 inEndRow );

			void						RecalcImageSize();
	//@}

	//! @name Sorting
	//@{
	static	bool						compare(
												CP_TV_DataBrowserRow *inDataBrowserCellDataL,
												CP_TV_DataBrowserRow *inDataBrowserCellDataR );
	//@}

public:
	//! @name Internal - called from Rows
	//@{
			void						InsertChildren_internal( CP_TV_DataBrowserRow *inParentRow );
			void						RemoveChildren_internal( CP_TV_DataBrowserRow *inParentRow );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CPLAT::CP_EventHandler& inEventHandler );

	virtual	void 						HandleControlHitEvent( CPLAT::CP_Event_Base& inEvent );
	//@}

	// class data
protected:
	CP_S_TV_DB_Properties				fProperties;

	CP_Rows_Array						*fRootRows;					/*! List of rows in the table (which are root). */
	CP_Rows_Array						*fDisplayRows;				/*! List of rows in the table currently being displayed. */
	
	CP_DataBrowserColumn_Array			*fColumnsInfo;
	CP_ContentHandlers_Array			*fContentHandlers;

	CP_TV_DataBrowser_CH_Base			*fBackgroundContentHandler;

	CP_InPlaceEditField					*fInPlaceEditField;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Events
	//@{
	virtual	ControlPartCode				HandleTracking(
												const CP_Point& inMouseLocation,
												ControlActionUPP actionProc );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Events
	//@{
		void							HandleDoubleClickEvent( CPLAT::CP_Event_Base& inEvent );
	//@}
		
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TV_DataBrowser

