//=========================================================================
//
//	FILE:				CP_TextView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms edit field control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 254 $
//						Latest Checkout:  $Date: 2005-08-22 21:49:19 -0700 (Mon, 22 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_TextView
#define _H_CP_TextView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_InStream.h"
#include "CP_M_CommandHandler.h"
#include "CP_View_Imp_GtkText.h"
#include "CP_View_Imp_Gtk2Text.h"
#include "CP_View_Imp_HITextView.h"
#include "CP_View_Imp_MLTE.h"
#include "CP_View_Imp_Rich.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_TextView_Base;

/*!
*	@class		CP_TextView
*	@brief		Implementation of a platforms text view.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_TextView : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TextView();
										CP_TextView(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false,
												bool inDrawFocusBorder = true );
	//@}
	//! Destructor.
	virtual								~CP_TextView() throw();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Appearance
	//@{
	virtual	void						Activate();
	virtual	void						Deactivate();
	//@}

	//! @name Files
	//@{
			OSStatus					ReadTextFromFile( const CP_FileSystemSpec& inFileSpec );
	//@}

	//! @name Getters
	//@{
			CP_String					GetText( char inLineEnd = CP_LF );
			UInt32						GetTextLength();
	//@}
						
	//! @name Misc
	//@{
	virtual	void						TextChanged();
	//@}

	//! @name Printing
	//@{
	virtual	void						Print();
	//@}

	//! @name Selection
	//@{
			void						GetSelection(
												UInt32 *inStartSelection,
												UInt32 *inEndSelection );
			void						SetSelection(
												UInt32 inStartSelection,
												UInt32 inEndSelection );
			bool						HasSelection();
	//@}

	//! @name Setters
	//@{
			void						SetText( const CP_String& inText );
			void						SetText( CP_InStream *inStream );
			void						InsertText( const CP_String& inText );
			void						InsertText(
												const char *inTextPtr,
												SInt32 inTextLength );
	virtual	void						SetFontSpec( CP_FontSpec& inFontSpec );

			void						SetFontName( const CP_String& inFontName );
			void						SetFontSize( SInt16 inSize );
			void						SetFontStyle( SInt16 inStyle );
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Events
	//@{
	virtual	void						OnLosingFocus();
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

private:
			void						CommandUpdateStatus( CP_Event_Base& inEvent );
			void						CommandProcess( CP_Event_Base& inEvent );

			void						HandleCommandStatus(
												CP_CommandStatus& inCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommandNo );

			void						HandleCutCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
			void						HandleCopyCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
			void						HandleClearCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
			void						HandlePasteCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
			void						HandleSelectAllCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
			void						HandleUndoCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
			void						HandleRedoCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );

	// class data
protected:
		// Signals
	CP_DataValueSignal					fValueChangedSignal;	/*! Signal sent when the value changes. */

	#if TARGET_OS_MAC
		#if TARGET_API_MAC_CARBON && TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_3) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3)
			CP_View_Imp_HITextView		*fTextViewImp;
			#if CP_UseDepreciated_10_4_APIs
				CP_View_Imp_MLTE		*fTextViewImpMLTE;
			#endif //CP_UseDepreciated_10_4_APIs
		#else
			CP_View_Imp_MLTE			*fTextViewImp;
		#endif
	#elif TARGET_OS_WIN32
		CP_View_Imp_Rich				*fTextViewImp;
	#elif TARGET_OS_LINUX
		#if TARGET_API_GTK
			CP_View_Imp_GtkText			*fTextViewImp;
		#else
			CP_View_Imp_Gtk2Text		*fTextViewImp;
		#endif
	#endif
	
	CP_FontSpec							fCurrentFontSpec;	/*! Contains the current font settings for the text. */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false,
												bool inDrawFocusBorder = true );
	//@}

	//! @name Events
	//@{
			void						HandleMouseDown( CP_Event_Base& inEvent );
	//@}
	
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false,
												bool inDrawFocusBorder = true );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												bool inHasHorizontalScroll = true,
												bool inHasVerticalScroll = true,
												bool inHasGrowBox = true,
												bool inReadOnly = false,
												bool inDrawFocusBorder = true );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TextView

