//=========================================================================
//
//	FILE:				CP_Toolbar.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms slider control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 255 $
//						Latest Checkout:  $Date: 2005-08-23 18:12:27 -0700 (Tue, 23 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Toolbar
#define _H_CP_Toolbar
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Structure_Defines.h"
#include "CP_View.h"
#include "CP_View_Imp_GtkToolbar.h"
#include "CP_View_Imp_HIToolbar.h"
#include "CP_View_Imp_Toolbar.h"
#include "CP_View_Imp_WinToolbar.h"

CPLAT_Begin_Namespace_CPLAT

	// toolbar defines
const SInt16 kCP_SpaceBetweenImageAndText								= 1;					
const SInt16 kCP_SpaceBetweenToolbarEdgeAndItem							= 6;					
const SInt16 kCP_HorizontalSpaceBetweenItems							= 6;					
const SInt16 kCP_ChevronWidth											= 35;					

/*!
*	@class		CP_Toolbar
*	@brief		Implementation of a platforms slider control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Toolbar : public CP_View
{
public:
	enum
	{
			// icon sizes
		CP_E_ToolbarItemIconSize_16x16,
		CP_E_ToolbarItemIconSize_24x24,
		CP_E_ToolbarItemIconSize_32x32,
		CP_E_ToolbarItemIconSize_48x48,
		CP_E_ToolbarItemIconSize_128x128
	};
	
	enum
	{
			// toolbar display flags
		CP_E_ToolbarShowTextOnly = 1,
		CP_E_ToolbarShowIconOnly = 2,
		CP_E_ToolbarShowIconAndText = 4
	};

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Toolbar();
										CP_Toolbar(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_Toolbar() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Signals
	//@{
			CP_DataValueSignal			GetAllowableItemsSignal()								{ return( fGetAllowableItemsSignal ); }
			CP_DataValueSignal			GetDefaultItemsSignal()									{ return( fGetDefaultItemsSignal ); }
			CP_DataValueSignal			GetCreateItemFromIdentifierSignal()						{ return( fCreateItemFromIdentifierSignal ); }
	//@}
		
	//! @name Toolbar Items
	//@{
			void						AddItem(
												const CP_String& inItemText,
												const CP_S_ViewContentInfo& inContentInfo,
												const CP_CommandNo inCommand,
												const CP_String& inHelpString = "",
												const CP_String& inSecondaryHelpString = "" );
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	// class data
protected:
	#if TARGET_OS_MAC
		#if TARGET_API_MAC_CARBON && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1020
			CP_View_Imp_HIToolbar		*fToolbarViewImp;
		#else
			CP_View_Imp_Toolbar			*fToolbarViewImp;
		#endif
	#elif TARGET_OS_WIN32
		CP_View_Imp_WinToolbar			*fToolbarViewImp;
	#elif TARGET_OS_LINUX
		CP_View_Imp_GtkToolbar			*fToolbarViewImp;
	#endif

private:
		// Signals
	CP_DataValueSignal					fGetAllowableItemsSignal;			/*! Signal sent when the toolbar requests a list of allowable items. */
	CP_DataValueSignal					fGetDefaultItemsSignal;				/*! Signal sent when the toolbar requests a list of default items. */
	CP_DataValueSignal					fCreateItemFromIdentifierSignal;	/*! Signal sent when the toolbar requests an item from an identifier. */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Toolbar
