//=========================================================================
//
//	FILE:				CP_Window.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for windows that display various panes and
//						other UI elements in them.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 71 $
//						Latest Checkout:  $Date: 2005-01-18 16:32:13 -0800 (Tue, 18 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_Window
#define _H_CP_Window
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_Persist.h"
#include "CP_Window_Imp_CE.h"			// Window implementation class for MacOS CarbonEvents
#include "CP_Window_Imp_Win.h"			// Window implementation class for Windows

#if TARGET_OS_LINUX
	#include "CP_Window_Imp_Gtk.h"
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Window
*	@brief		Base class for windows that display various panes and other UI elements in them.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Window : public CP_Window_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Window();
	//! Destructor.
	virtual								~CP_Window() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Initialization
	//@{
			void						Create(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inWindowTitle,
												const CP_WindowTypeBase& inWindowType );
	//@}

	//! @name Appearance
	//@{
	virtual	void						Show();
	virtual	void						Hide();
	//@}

	//! @name Getters
	//@{
											// Getters
										/*!
										*	Return the underlying OSs platform window object.
										*	
										*	@exception	none
										*	
										*	@return	OSWindow const			Underlying platforms window object.
										*/
			OSWindow					GetOSWindow() const									{ return( GetImpPlatformWindow() ); }

	virtual	CP_Window					*GetContainingWindow();
	//@}

	//! @name Positioners
	//@{
			void						SetLocation( const CP_Point& inLocation );
			void						SetSize( const CP_Size& inSize );
	//@}

	//! @name Setters
	//@{
											// Setters
										/*!
										*	Sets the underlying OSs platform window object.
										*	
										*	@exception	none
										*
										*	@return		void.
										*/
	inline	void						SetOSWindow( OSWindow inWindow )					{ SetImpPlatformWindow( inWindow ); }
	//@}

protected:
	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}
	
	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
										CP_Window( WindowRef inOSWindow );
#endif

#if TARGET_OS_WIN32
public:
	virtual CP_RGBColor					GetBackColor();
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Window

