//=========================================================================
//
//	FILE:				CP_View_Imp_Win.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Windows implementation of a CP_View.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 253 $
//						Latest Checkout:  $Date: 2005-08-22 20:25:19 -0700 (Mon, 22 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_View_Imp_Win
#define _H_CP_View_Imp_Win
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_EventHandler.h"
#include "CP_M_Persist.h"
#include "CP_View_Imp_Base.h"
#include "CP_Point.h"
#include "CP_Region.h"
#include "CP_Size.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_View_Imp_Win
*	@brief		Windows implementation of the CP_View class.
*/
class CP_EXPORT CP_View_Imp_Win : public CP_View_Imp_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_View_Imp_Win(
												CP_View_Imp *inOwnerViewImp,
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inCreateImp = false );
	//! Destructor.
	virtual								~CP_View_Imp_Win();
	//@}
			
	//! @name Appearance
	//@{
	virtual	void						Show();
	virtual	void						Hide();

	virtual	void						Enable();
	virtual	void						Disable();
	//@}	

	//! @name Drawing
	//@{
	virtual	CP_Canvas					SetupForDrawing();
	virtual	void						Draw();
	virtual	void						Refresh();
	virtual void						FinishDrawing( CP_Canvas& drawCanvas );
	//@}

	//! @name Drawing
	//@{
	virtual	CP_Rect						GetPortBounds();
	//@}

	//! @name Implementation
	//@{
			OSWindow					CreateChildWindow(
												const CP_String& inTitle,
												const CP_String& inWindowType,
												DWORD inWindowFlags,
												DWORD inExWindowFlags,
												const CP_Point& inUpperLeft,
												const CP_Size& inSize );
			void						CreateWindowsChildWindow(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Custom Pane
	//@{
			bool						GetIsCustomPane()												{ return( fIsCustomPane ); }
			void						SetIsCustomPane( bool inIsCustomPane )							{ fIsCustomPane = inIsCustomPane; }
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );

			LRESULT						HandleWinProcMessage(
												HWND hwnd,
												UINT iMsg,
												WPARAM wParam,
												LPARAM lParam );

			void						HandleMouseEnteredEvent( CP_Event_Base& inEvent );
			void						HandleMouseExitedEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Getters
	//@{
	virtual	CP_EventHandler&			GetEventHandler()									{ return( fEventHandler ); }
			WNDPROC						GetOldWinProc()										{ return( fnOldWndProc ); }
			HBRUSH						GetBackgroundBrush()								{ return( fsBackgroundBrush ); }

	static CP_View_Imp_Win				*GetLastViewWithMouse()								{ return( fsLastViewWithMouse ); }
	static void							SetLastViewWithMouse( CP_View_Imp_Win *inView )		{ fsLastViewWithMouse = inView; }
	//@}

	//! @name Positioning
	//@{
	virtual	void						Move( const CP_Point& inOffset );
	//@}

	//! @name Setters
	//@{
			void						SetOldWinProc( WNDPROC inOldWndProc );
	//@}

	//! @name WinViewID
	//@{
			SInt16						GetWinViewID()										{ return( fsWinViewID ); }
			void						SetWinViewID( SInt16 inWinViewID )					{ fsWinViewID = inWinViewID; }
	//@}

	//! @name Testers
	//@{
	virtual	bool						IsVisible()	const;
	virtual	bool						IsActive() const;
	virtual	bool						IsEnabled() const;
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_View_Imp_Win();

	//! @name Drawing
	//@{
			void						Invalidate( const CP_Region& inInvalidArea );
	virtual	void						HandleErase( CP_Event_Base& inEvent );
	//@}

	//! @name Implementation
	//@{
			void						CreateViewChildWindow(
												DWORD inExWindowFlags,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Registration
	//@{
			void 						RegisterWindowClass();
	//@}

	//! @name Positioning
	//@{
	virtual	void						SetPortBounds( const CP_Rect& inBounds );
	virtual	void						SetLocation( const CP_Point& inLocation );
	virtual	void						SetSize( const CP_Size& inSize );
	//@}

private:

	//! @name Events
	//@{
			void						HandleDraw( CP_Event_Base& inEvent );


			void						HandleMouseMovedEvent( CP_Event_Base& inEvent );
	//@}

	// class data
protected:
	CP_EventHandler						fEventHandler;		/*! Class that handles all of the events for this windnow. */
	CP_Point							fLocation;
								
	UInt32								fWinStyleFlags;		/*! The Windows style flags used to create the pane. */
	UInt32								fWinExtStyleFlags;	/*! The Windows extended style flags used to create the pane. */
	WNDPROC								fnOldWndProc;		/*! The standard Windows window callback. This is the one returned before
																we set the callback to our panes callback. */

	static SInt16						fsWinViewID;		/*! View id of this viewe under Windows. */
	bool								fIsCustomPane;		/*! Is this view a custom pane, and not a Windows control. */

	static HBRUSH						fsBackgroundBrush;	// brush object used for background


	static CP_View_Imp_Win				*fsLastViewWithMouse;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_WIN32

#endif	// _H_CP_View_Imp_Win

